=== modified file 'Makefile'
--- Makefile	2007-12-11 02:07:30 +0000
+++ Makefile	2007-12-17 01:33:01 +0000
@@ -75,7 +75,7 @@
 ifdef PRETTY
 rst2html := $(PYTHON) tools/rst2prettyhtml.py doc/bazaar-vcs.org.kid 
 else
-rst2html := $(PYTHON) tools/rst2html.py --link-stylesheet --footnote-references=superscript 
+rst2html := $(PYTHON) tools/rst2html.py --link-stylesheet --footnote-references=superscript --halt=warning
 endif
 
 # translate txt docs to html
@@ -99,7 +99,7 @@
 dev_htm_files := $(patsubst %.txt, %.html, $(dev_txt_files)) 
 
 doc/en/user-guide/index.html: $(wildcard $(addsuffix /*.txt, doc/en/user-guide)) 
-	$(rst2html) --stylesheet=../../default.css $< $@
+	$(rst2html) --stylesheet=../../default.css doc/en/user-guide/index.txt $@
 
 doc/developers/%.html: doc/developers/%.txt
 	$(rst2html) --stylesheet=../default.css $< $@
@@ -115,6 +115,7 @@
 		 bzrlib/conflicts.py \
 		 bzrlib/help_topics.py \
 		 bzrlib/sign_my_commits.py \
+		 bzrlib/bugtracker.py \
 		 generate_docs.py \
 		 tools/doc_generate/__init__.py \
 		 tools/doc_generate/autodoc_man.py \

=== modified file 'NEWS'
--- NEWS	2007-12-10 02:52:06 +0000
+++ NEWS	2007-12-17 01:33:01 +0000
@@ -13,15 +13,42 @@
 
   IMPROVEMENTS:
 
+   * ``bzr pack`` now orders revision texts in topological order, with newest
+     at the start of the file, promoting linear reads for ``bzr log`` and the
+     like. This partially fixes #154129. (Robert Collins)
+
+   * Merge directives now fetch prerequisites from the target branch if
+     needed.  (Aaron Bentley)
+
+   * ``reconfigure`` can now convert from repositories.  (Aaron Bentley)
+
+   * ``-l`` is now a short form for ``--limit`` in ``log``.  (Matt Nordhoff)
+
+  BUGFIXES:
+
+   * Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
+
+   * Transform failures no longer cause ImmortalLimbo errors (Aaron Bentley,
+     #137681)
+
+   * ``uncommit`` works even when the commit messages of revisions to be
+     removed use characters not supported in the terminal encoding.
+     (Aaron Bentley)
+
+   * When dumb http servers return whole files instead of the requested ranges,
+     read the remaining bytes by chunks to avoid overflowing network buffers.
+     (Vincent Ladeuil, #175886)
+
+   * ``reconfigure`` is able to convert trees into lightweight checkouts.
+     (Aaron Bentley)
+
+  DOCUMENTATION:
+
    * Reference material has now be moved out of the User Guide and added
      to the User Reference. The User Reference has gained 4 sections as
      a result: Authenication Settings, Configuration Settings, Conflicts
      and Hooks. (Ian Clatworthy)
 
-  BUGFIXES:
-
-   * Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
-
   INTERNALS:
 
     * Help topics can now be loaded from files. 
@@ -32,7 +59,24 @@
   TESTING:
 
 
-bzr 1.0rc2 2007-12-07
+bzr 1.0 2007-12-14
+==================
+
+  DOCUMENTATION:
+
+   * More improvements and fixes to the User Guide.  (Ian Clatworthy)
+
+   * Add information on cherrypicking/rebasing to the User Guide.
+     (Ian Clatworthy)
+
+   * Improve bug tracker integration documentation. (Ian Clatworthy)
+
+   * Minor edits to ``Bazaar in five minutes`` from David Roberts and
+     to the rebasing section of the User Guide from Aaron Bentley.
+     (Ian Clatworthy)
+
+
+bzr 1.0rc3 2007-12-11
 =====================
 
   CHANGES:
@@ -42,6 +86,37 @@
      by mail to the mailing list.
      (Martin Pool)
 
+  BUGFIXES:
+
+   * Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
+
+   * Give more feedback during long http downloads by making readv deliver data
+     as it arrives for urllib, and issue more requests for pycurl. High latency
+     networks are better handled by urllib, the pycurl implementation give more
+     feedback but also incur more latency.
+     (Vincent Ladeuil, #173010)
+
+   * Implement _make_parents_provider on RemoteRepository, allowing generating
+     bundles against branches on a smart server.  (Andrew Bennetts, #147836)
+
+  DOCUMENTATION:
+
+   * Improved user guide.  (Ian Clatworthy)
+
+   * The single-page quick reference guide is now available as a PDF.
+     (Ian Clatworthy)
+
+  INTERNALS:
+
+    * readv urllib http implementation is now a real iterator above the
+      underlying socket and deliver data as soon as it arrives. 'get' still
+      wraps its output in a StringIO.
+      (Vincent Ladeuil)
+
+
+bzr 1.0rc2 2007-12-07
+=====================
+
   IMPROVEMENTS:
 
    * Added a --coverage option to selftest. (Andrew Bennetts)
@@ -101,16 +176,12 @@
    * Fall back to showing e-mail in ``log --short/--line`` if the 
      committer/author has only e-mail. (Lukáš Lalinský, #157026)
 
-   * Implement _make_parents_provider on RemoteRepository, allowing generating
-     bundles against branches on a smart server.  (Andrew Bennetts, #147836)
-
   API BREAKS:
 
    * Deprecate not passing a ``location`` argument to commit reporters'
      ``started`` methods. (Matt Nordhoff)
 
 
-
 bzr 1.0rc1 2007-11-30
 =====================
 

=== modified file 'bzr'
--- bzr	2007-11-02 08:45:10 +0000
+++ bzr	2007-12-14 07:35:49 +0000
@@ -21,6 +21,8 @@
 import os
 import sys
 
+# update this on each release
+_script_version = (1, 1, 0)
 
 if __doc__ is None:
     print "bzr does not support python -OO."
@@ -67,7 +69,7 @@
                      "\n")
     raise
 
-if bzrlib.version_info[:3] != (0, 93, 0):
+if bzrlib.version_info[:3] != _script_version:
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2007-11-30 03:49:39 +0000
+++ bzrlib/__init__.py	2007-12-14 07:35:49 +0000
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 93, 0, 'dev', 0)
+version_info = (1, 1, 0, 'dev', 0)
 
 # API compatibility version: bzrlib is currently API compatible with 0.18.
 api_minimum_version = (0, 18, 0)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-12-07 05:46:51 +0000
+++ bzrlib/branch.py	2007-12-10 19:20:41 +0000
@@ -1794,22 +1794,6 @@
         # last_rev is not in the other_last_rev history, AND
         # other_last_rev is not in our history, and do it without pulling
         # history around
-        last_rev = _mod_revision.ensure_null(self.last_revision())
-        if last_rev != _mod_revision.NULL_REVISION:
-            other.lock_read()
-            try:
-                other_last_rev = other.last_revision()
-                if not _mod_revision.is_null(other_last_rev):
-                    # neither branch is new, we have to do some work to
-                    # ascertain diversion.
-                    remote_graph = other.repository.get_revision_graph(
-                        other_last_rev)
-                    local_graph = self.repository.get_revision_graph(last_rev)
-                    if (last_rev not in remote_graph and
-                        other_last_rev not in local_graph):
-                        raise errors.DivergedBranches(self, other)
-            finally:
-                other.unlock()
         self.set_bound_location(other.base)
 
     @needs_write_lock

=== modified file 'bzrlib/bugtracker.py'
--- bzrlib/bugtracker.py	2007-11-27 19:27:40 +0000
+++ bzrlib/bugtracker.py	2007-12-13 13:33:36 +0000
@@ -39,6 +39,71 @@
 """
 
 
+_bugs_help = \
+"""When making a commit, metadata about bugs fixed by that change can be
+recorded by using the --fixes option. For each bug marked as fixed, an
+entry is included in the 'bugs' revision property stating '<url> <status>'.
+Support for Launchpad's central bug tracker is built in. For other bug
+trackers, configuration is required in advance so that the correct URL
+can be recorded.
+
+In addition to Launchpad, Bazaar directly supports the generation of
+URLs appropriate for Bugzilla and Trac. If your project uses a different
+bug tracker, it is easy to add support for it by writing a plugin, say.
+If you use Bugzilla or Trac, then you only need to set a configuration
+variable which contains the base URL of the bug tracker. These options
+can go into ``bazaar.conf``, ``branch.conf`` or into a branch-specific
+configuration section in ``locations.conf``.  You can set up these values
+for each of the projects you work on.
+
+Note: As you provide a short name for each tracker, you can specify one or
+more bugs in one or more trackers at commit time if you wish.
+
+bugzilla_<tracker_abbreviation>_url
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If present, the location of the Bugzilla bug tracker referred to by
+<tracker_abbreviation>. This option can then be used together with ``bzr commit
+--fixes`` to mark bugs in that tracker as being fixed by that commit. For
+example::
+
+    bugzilla_squid_url = http://www.squid-cache.org/bugs
+
+would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as
+fixed.
+
+trac_<tracker_abbrevation>_url
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If present, the location of the Trac instance referred to by
+<tracker_abbreviation>. This option can then be used together with ``bzr commit
+--fixes`` to mark bugs in that tracker as being fixed by that commit. For
+example::
+
+    trac_twisted_url = http://www.twistedmatrix.com/trac
+
+would allow ``bzr commit --fixes twisted:1234`` to mark Twisted's bug 1234 as
+fixed.
+
+bugtracker_<tracker_abbrevation>_url
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+If present, the location of a generic bug tracker instance referred to by
+<tracker_abbreviation>. The location must contain an ``{id}`` placeholder,
+which will be replaced by a specific bug ID. This option can then be used
+together with ``bzr commit --fixes`` to mark bugs in that tracker as being
+fixed by that commit. For example::
+
+    bugtracker_python_url = http://bugs.python.org/issue{id}
+
+would allow ``bzr commit --fixes python:1234`` to mark bug 1234 in Python's
+Roundup bug tracker as fixed, or::
+
+    bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}
+
+for CPAN's RT bug tracker.
+"""
+
+
 def get_bug_url(abbreviated_bugtracker_name, branch, bug_id):
     """Return a URL pointing to the canonical web page of the bug identified by
     'bug_id'.
@@ -65,13 +130,7 @@
                                                    branch)
 
     def help_topic(self, topic):
-        return textwrap.dedent("""\
-        Bazaar provides the ability to store information about bugs being fixed
-        as metadata on a revision.
-
-        For each bug marked as fixed, an entry is included in the 'bugs'
-        revision property stating '<url> <status>'.
-        """)
+        return _bugs_help
 
 
 tracker_registry = TrackerRegistry()

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-12-09 06:35:14 +0000
+++ bzrlib/builtins.py	2007-12-15 16:35:40 +0000
@@ -1666,6 +1666,7 @@
                         'regular expression.',
                    type=str),
             Option('limit',
+                   short_name='l',
                    help='Limit the output to the first N revisions.',
                    argname='N',
                    type=_parse_limit),
@@ -2231,7 +2232,7 @@
                     "files in the working tree."),
              ListOption('fixes', type=str,
                     help="Mark a bug as being fixed by this revision."),
-             Option('author', type=str,
+             Option('author', type=unicode,
                     help="Set the author's name, if it's different "
                          "from the committer."),
              Option('local',
@@ -3579,6 +3580,7 @@
                     Option('force', help='Say yes to all questions.')]
     takes_args = ['location?']
     aliases = []
+    encoding_type = 'replace'
 
     def run(self, location=None,
             dry_run=False, verbose=False,
@@ -4025,17 +4027,17 @@
     def _run(self, submit_branch, revision, public_branch, remember, format,
              no_bundle, no_patch, output, from_, mail_to, message):
         from bzrlib.revision import NULL_REVISION
+        branch = Branch.open_containing(from_)[0]
         if output is None:
             outfile = StringIO()
         elif output == '-':
             outfile = self.outf
         else:
             outfile = open(output, 'wb')
+        # we may need to write data into branch's repository to calculate
+        # the data to send.
+        branch.lock_write()
         try:
-            branch = Branch.open_containing(from_)[0]
-            # we may need to write data into branch's repository to calculate
-            # the data to send.
-            branch.lock_write()
             if output is None:
                 config = branch.get_config()
                 if mail_to is None:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-12-04 18:11:51 +0000
+++ bzrlib/errors.py	2007-12-15 16:04:18 +0000
@@ -653,7 +653,7 @@
 
 class FileInWrongBranch(BzrError):
 
-    _fmt = 'File "%(path)s" in not in branch %(branch_base)s.'
+    _fmt = 'File "%(path)s" is not in branch %(branch_base)s.'
 
     def __init__(self, branch, path):
         BzrError.__init__(self)
@@ -1455,11 +1455,10 @@
 
 class InvalidRange(TransportError):
 
-    _fmt = "Invalid range access in %(path)s at %(offset)s."
-    
-    def __init__(self, path, offset):
-        TransportError.__init__(self, ("Invalid range access in %s at %d"
-                                       % (path, offset)))
+    _fmt = "Invalid range access in %(path)s at %(offset)s: %(msg)s"
+
+    def __init__(self, path, offset, msg=None):
+        TransportError.__init__(self, msg)
         self.path = path
         self.offset = offset
 
@@ -1476,7 +1475,7 @@
 class InvalidHttpRange(InvalidHttpResponse):
 
     _fmt = "Invalid http range %(range)r for %(path)s: %(msg)s"
-    
+
     def __init__(self, path, range, msg):
         self.range = range
         InvalidHttpResponse.__init__(self, path, msg)
@@ -1485,7 +1484,7 @@
 class InvalidHttpContentType(InvalidHttpResponse):
 
     _fmt = 'Invalid http Content-type "%(ctype)s" for %(path)s: %(msg)s'
-    
+
     def __init__(self, path, ctype, msg):
         self.ctype = ctype
         InvalidHttpResponse.__init__(self, path, msg)

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2007-12-11 13:29:00 +0000
+++ bzrlib/help_topics/__init__.py	2007-12-17 01:33:01 +0000
@@ -615,8 +615,9 @@
                         "Help on status flags")
 def get_bugs_topic(topic):
     from bzrlib import bugtracker
-    return "Bug Trackers\n\n" + bugtracker.tracker_registry.help_topic(topic)
-topic_registry.register('bugs', get_bugs_topic, 'Bug tracker support')
+    return "Bug Tracker Settings\n\n" + \
+        bugtracker.tracker_registry.help_topic(topic)
+topic_registry.register('bugs', get_bugs_topic, 'Bug tracker settings')
 topic_registry.register('env-variables', _env_variables,
                         'Environment variable names and values')
 topic_registry.register('files', _files,

=== modified file 'bzrlib/help_topics/en/authentication.txt'
--- bzrlib/help_topics/en/authentication.txt	2007-12-09 22:35:10 +0000
+++ bzrlib/help_topics/en/authentication.txt	2007-12-17 01:33:01 +0000
@@ -6,7 +6,7 @@
 ------
 
 Many different authentication policies can be described in the
-``$HOME/.bazaar/authentication.conf`` file but a particular user should need
+``authentication.conf`` file but a particular user should need
 only a few definitions to cover his needs without having to specify a user and
 a password for every branch he uses.
 
@@ -112,7 +112,7 @@
 The general rules for `configuration files`_ apply except for the variable
 policies.
 
-.. _configuration files: configuration.html
+.. _configuration files: #configuration-settings
 
 Each section describes an authentication definition.
 

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2007-12-09 22:35:10 +0000
+++ bzrlib/help_topics/en/configuration.txt	2007-12-17 01:33:01 +0000
@@ -1,8 +1,6 @@
 Configuration Settings
 =======================
 
-.. Information on how to configure Bazaar.
-
 .. TODO: Should have some explanation of why you'd want things in
 .. branch.conf.
 
@@ -31,8 +29,9 @@
 ~~~~~~~~~~~~~~~
 
 Control whether SIGQUIT behaves normally or invokes a breakin debugger.
-0 = Standard SIGQUIT behavior
-1 = Invoke breakin debugger (default)
+
+* 0 = Standard SIGQUIT behavior (normally, exit with a core dump)
+* 1 = Invoke breakin debugger (default)
 
 BZR_HOME
 ~~~~~~~~
@@ -48,15 +47,16 @@
 ~~~~~~~
 
 Control whether to launch a debugger on error.
-0 = Standard behavior
-1 = Launch debugger
+
+* 0 = Standard behavior
+* 1 = Launch debugger
 
 BZR_REMOTE_PATH
 ~~~~~~~~~~~~~~~
 
 Path to the Bazaar executable to use when using the bzr+ssh protocol.
 
-See also the ``bzr_remote_path`` configuration value
+See also the ``bzr_remote_path`` configuration value.
 
 BZR_EDITOR
 ~~~~~~~~~~
@@ -80,8 +80,12 @@
 Location
 ~~~~~~~~
 
-Configuration files are located in ``$HOME/.bazaar`` and are
-sometimes referred to as ``ini files``:
+Configuration files are located in ``$HOME/.bazaar`` on Linux/Unix and
+``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` on
+Windows. (You can check the location for your system by using
+``bzr version``.)
+
+There are three primary configuration files in this location:
 
 * ``bazaar.conf`` describes default configuration options,
 
@@ -174,7 +178,7 @@
 The main configuration file, bazaar.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-The main configuration file, ``$HOME/.bazaar/bazaar.conf``, only allows one
+``bazaar.conf`` only allows one
 section called ``[DEFAULT]``. This default section contains the default
 configuration options for all branches. The default section can be
 overriden by providing a branch-specific section in ``locations.conf``.
@@ -191,7 +195,7 @@
 The branch location configuration file, locations.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-``$HOME/.bazaar/locations.conf`` allows one to specify overriding settings for
+``locations.conf`` allows one to specify overriding settings for
 a specific branch. The format is almost identical to the default section in
 bazaar.conf with one significant change: The section header, instead of saying
 default, will be the path to a branch that you wish to override a value
@@ -210,7 +214,7 @@
 The authentication configuration file, authentication.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-``$HOME/.bazaar/authentication.conf`` allows one to specify credentials for
+``authentication.conf`` allows one to specify credentials for
 remote servers. This can be used for all the supported transports and any part
 of bzr that requires authentication (smtp for example).
 
@@ -218,9 +222,8 @@
 variable policies which don't apply.
 
 For more information on the possible uses of the authentication configuration
-file see the `authentication configuration file documentation`_.
+file see `Authentication Settings`_.
 
-.. _authentication configuration file documentation: authentication_conf.html
 
 Common variable options
 -----------------------
@@ -238,7 +241,8 @@
 
 The path of the editor that you wish to use if *bzr commit* is run without
 a commit message. This setting is trumped by the environment variable
-``$BZR_EDITOR``, and overrides ``$VISUAL`` and ``$EDITOR``.
+``BZR_EDITOR``, and overrides the ``VISUAL`` and ``EDITOR`` environment
+variables.
 
 check_signatures
 ~~~~~~~~~~~~~~~~
@@ -358,10 +362,11 @@
 not publically-accessible).  Used (and set) by ``bzr send``.
 
 
-Branch 6 options
-----------------
+Branch type specific options
+----------------------------
 
-These options apply only to branches that use the "dirstate-tags" format.  They
+These options apply only to branches that use the ``dirstate-tags`` or
+later format.  They
 are usually set in ``.bzr/branch/branch.conf`` automatically, but may be
 manually set in ``locations.conf`` or ``bazaar.conf``.
 
@@ -397,4 +402,3 @@
 
 If set to "True", the branch should act as a checkout, and push each commit to
 the bound_location.  This option is normally set by ``bind``/``unbind``.
-

=== modified file 'bzrlib/merge_directive.py'
--- bzrlib/merge_directive.py	2007-08-15 20:49:22 +0000
+++ bzrlib/merge_directive.py	2007-12-14 04:44:58 +0000
@@ -190,7 +190,32 @@
                     StringIO(self.get_raw_bundle()))
                 # We don't use the bundle's target revision, because
                 # MergeDirective.revision_id is authoritative.
-                info.install_revisions(target_repo, stream_input=False)
+                try:
+                    info.install_revisions(target_repo, stream_input=False)
+                except errors.RevisionNotPresent:
+                    # At least one dependency isn't present.  Try installing
+                    # missing revisions from the submit branch
+                    submit_branch = _mod_branch.Branch.open(self.target_branch)
+                    missing_revisions = []
+                    bundle_revisions = set(r.revision_id for r in
+                                           info.real_revisions)
+                    for revision in info.real_revisions:
+                        for parent_id in revision.parent_ids:
+                            if (parent_id not in bundle_revisions and
+                                not target_repo.has_revision(parent_id)):
+                                missing_revisions.append(parent_id)
+                    # reverse missing revisions to try to get heads first
+                    unique_missing = []
+                    unique_missing_set = set()
+                    for revision in reversed(missing_revisions):
+                        if revision in unique_missing_set:
+                            continue
+                        unique_missing.append(revision)
+                        unique_missing_set.add(revision)
+                    for missing_revision in unique_missing:
+                        target_repo.fetch(submit_branch.repository,
+                                          missing_revision)
+                    info.install_revisions(target_repo, stream_input=False)
             else:
                 source_branch = _mod_branch.Branch.open(self.source_branch)
                 target_repo.fetch(source_branch.repository, self.revision_id)

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2007-11-27 19:37:36 +0000
+++ bzrlib/reconfigure.py	2007-12-13 16:38:54 +0000
@@ -62,8 +62,6 @@
 
         :param bzrdir: The bzrdir to reconfigure
         :raise errors.AlreadyBranch: if bzrdir is already a branch
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=False, want_branch=True,
@@ -78,8 +76,6 @@
 
         :param bzrdir: The bzrdir to reconfigure
         :raise errors.AlreadyTree: if bzrdir is already a tree
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
@@ -95,8 +91,6 @@
         :param bzrdir: The bzrdir to reconfigure
         :param bound_location: The location the checkout should be bound to.
         :raise errors.AlreadyCheckout: if bzrdir is already a checkout
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = Reconfigure(bzrdir, bound_location)
         reconfiguration._plan_changes(want_tree=True, want_branch=True,
@@ -113,8 +107,6 @@
         :param bound_location: The location the checkout should be bound to.
         :raise errors.AlreadyLightweightCheckout: if bzrdir is already a
             lightweight checkout
-        :raise errors.ReconfigurationNotSupported: if bzrdir does not contain
-            a branch or branch reference
         """
         reconfiguration = klass(bzrdir, reference_location)
         reconfiguration._plan_changes(want_tree=True, want_branch=False,
@@ -130,17 +122,14 @@
             raise errors.ReconfigurationNotSupported(self.bzrdir)
         if want_branch and want_reference:
             raise errors.ReconfigurationNotSupported(self.bzrdir)
-        if (want_branch or want_reference) and (self.local_branch is None and
-                                                self.referenced_branch
-                                                is None):
-            raise errors.ReconfigurationNotSupported(self.bzrdir)
         if self.repository is None:
             if not want_reference:
                 self._create_repository = True
         else:
             if want_reference and (self.repository.bzrdir.root_transport.base
                                    == self.bzrdir.root_transport.base):
-                self._destroy_repository = True
+                if not self.repository.is_shared():
+                    self._destroy_repository = True
         if self.referenced_branch is None:
             if want_reference:
                 self._create_reference = True
@@ -168,7 +157,8 @@
         """Return True if changes are planned, False otherwise"""
         return (self._unbind or self._bind or self._destroy_tree
                 or self._create_tree or self._destroy_reference
-                or self._create_branch or self._create_repository)
+                or self._create_branch or self._create_repository
+                or self._create_reference)
 
     def _check(self):
         """Raise if reconfiguration would destroy local changes"""
@@ -223,18 +213,20 @@
             repo = self.bzrdir.create_repository()
         else:
             repo = self.repository
-        if self._create_branch:
+        if self._create_branch and self.referenced_branch is not None:
             repo.fetch(self.referenced_branch.repository,
                        self.referenced_branch.last_revision())
+        last_revision_info = None
         if self._destroy_reference:
-            reference_info = self.referenced_branch.last_revision_info()
+            last_revision_info = self.referenced_branch.last_revision_info()
             self.bzrdir.destroy_branch()
         if self._destroy_branch:
-            reference_info = self.local_branch.last_revision_info()
+            last_revision_info = self.local_branch.last_revision_info()
             self.bzrdir.destroy_branch()
         if self._create_branch:
             local_branch = self.bzrdir.create_branch()
-            local_branch.set_last_revision_info(*reference_info)
+            if last_revision_info is not None:
+                local_branch.set_last_revision_info(*last_revision_info)
         else:
             local_branch = self.local_branch
         if self._create_reference:

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2007-12-03 19:26:40 +0000
+++ bzrlib/repofmt/pack_repo.py	2007-12-14 07:35:49 +0000
@@ -610,8 +610,9 @@
         revision_nodes = self._pack_collection._index_contents(revision_index_map, revision_keys)
         # copy revision keys and adjust values
         self.pb.update("Copying revision texts", 1)
-        list(self._copy_nodes_graph(revision_nodes, revision_index_map,
-            self.new_pack._writer, self.new_pack.revision_index))
+        total_items, readv_group_iter = self._revision_node_readv(revision_nodes)
+        list(self._copy_nodes_graph(revision_index_map, self.new_pack._writer,
+            self.new_pack.revision_index, readv_group_iter, total_items))
         if 'pack' in debug.debug_flags:
             mutter('%s: create_pack: revisions copied: %s%s %d items t+%6.3fs',
                 time.ctime(), self._pack_collection._upload_transport.base,
@@ -639,8 +640,10 @@
         # XXX: Should be a helper function to allow different inv representation
         # at this point.
         self.pb.update("Copying inventory texts", 2)
-        inv_lines = self._copy_nodes_graph(inv_nodes, inventory_index_map,
-            self.new_pack._writer, self.new_pack.inventory_index, output_lines=True)
+        total_items, readv_group_iter = self._least_readv_node_readv(inv_nodes)
+        inv_lines = self._copy_nodes_graph(inventory_index_map,
+            self.new_pack._writer, self.new_pack.inventory_index,
+            readv_group_iter, total_items, output_lines=True)
         if self.revision_ids:
             self._process_inventory_lines(inv_lines)
         else:
@@ -675,8 +678,9 @@
                     a_missing_key[0])
         # copy text keys and adjust values
         self.pb.update("Copying content texts", 3)
-        list(self._copy_nodes_graph(text_nodes, text_index_map,
-            self.new_pack._writer, self.new_pack.text_index))
+        total_items, readv_group_iter = self._least_readv_node_readv(text_nodes)
+        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
+            self.new_pack.text_index, readv_group_iter, total_items))
         self._log_copied_texts()
 
     def _check_references(self):
@@ -787,8 +791,8 @@
                 pb.update("Copied record", record_index)
                 record_index += 1
 
-    def _copy_nodes_graph(self, nodes, index_map, writer, write_index,
-        output_lines=False):
+    def _copy_nodes_graph(self, index_map, writer, write_index,
+        readv_group_iter, total_items, output_lines=False):
         """Copy knit nodes between packs.
 
         :param output_lines: Return lines present in the copied data as
@@ -796,8 +800,8 @@
         """
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            for result in self._do_copy_nodes_graph(nodes, index_map, writer,
-                write_index, output_lines, pb):
+            for result in self._do_copy_nodes_graph(index_map, writer,
+                write_index, output_lines, pb, readv_group_iter, total_items):
                 yield result
         except Exception:
             # Python 2.4 does not permit try:finally: in a generator.
@@ -806,42 +810,21 @@
         else:
             pb.finished()
 
-    def _do_copy_nodes_graph(self, nodes, index_map, writer, write_index,
-        output_lines, pb):
+    def _do_copy_nodes_graph(self, index_map, writer, write_index,
+        output_lines, pb, readv_group_iter, total_items):
         # for record verification
         knit_data = _KnitData(None)
         # for line extraction when requested (inventories only)
         if output_lines:
             factory = knit.KnitPlainFactory()
-        # plan a readv on each source pack:
-        # group by pack
-        nodes = sorted(nodes)
-        # how to map this into knit.py - or knit.py into this?
-        # we don't want the typical knit logic, we want grouping by pack
-        # at this point - perhaps a helper library for the following code 
-        # duplication points?
-        request_groups = {}
         record_index = 0
-        pb.update("Copied record", record_index, len(nodes))
-        for index, key, value, references in nodes:
-            if index not in request_groups:
-                request_groups[index] = []
-            request_groups[index].append((key, value, references))
-        for index, items in request_groups.iteritems():
-            pack_readv_requests = []
-            for key, value, references in items:
-                # ---- KnitGraphIndex.get_position
-                bits = value[1:].split(' ')
-                offset, length = int(bits[0]), int(bits[1])
-                pack_readv_requests.append((offset, length, (key, value[0], references)))
-            # linear scan up the pack
-            pack_readv_requests.sort()
+        pb.update("Copied record", record_index, total_items)
+        for index, readv_vector, node_vector in readv_group_iter:
             # copy the data
             transport, path = index_map[index]
-            reader = pack.make_readv_reader(transport, path,
-                [offset[0:2] for offset in pack_readv_requests])
-            for (names, read_func), (_1, _2, (key, eol_flag, references)) in \
-                izip(reader.iter_records(), pack_readv_requests):
+            reader = pack.make_readv_reader(transport, path, readv_vector)
+            for (names, read_func), (key, eol_flag, references) in \
+                izip(reader.iter_records(), node_vector):
                 raw_data = read_func(None)
                 version_id = key[-1]
                 if output_lines:
@@ -868,6 +851,43 @@
         return text_index_map, self._pack_collection._index_contents(text_index_map,
             self._text_filter)
 
+    def _least_readv_node_readv(self, nodes):
+        """Generate request groups for nodes using the least readv's.
+        
+        :param nodes: An iterable of graph index nodes.
+        :return: Total node count and an iterator of the data needed to perform
+            readvs to obtain the data for nodes. Each item yielded by the
+            iterator is a tuple with:
+            index, readv_vector, node_vector. readv_vector is a list ready to
+            hand to the transport readv method, and node_vector is a list of
+            (key, eol_flag, references) for the the node retrieved by the
+            matching readv_vector.
+        """
+        # group by pack so we do one readv per pack
+        nodes = sorted(nodes)
+        total = len(nodes)
+        request_groups = {}
+        for index, key, value, references in nodes:
+            if index not in request_groups:
+                request_groups[index] = []
+            request_groups[index].append((key, value, references))
+        result = []
+        for index, items in request_groups.iteritems():
+            pack_readv_requests = []
+            for key, value, references in items:
+                # ---- KnitGraphIndex.get_position
+                bits = value[1:].split(' ')
+                offset, length = int(bits[0]), int(bits[1])
+                pack_readv_requests.append(
+                    ((offset, length), (key, value[0], references)))
+            # linear scan up the pack to maximum range combining.
+            pack_readv_requests.sort()
+            # split out the readv and the node data.
+            pack_readv = [readv for readv, node in pack_readv_requests]
+            node_vector = [node for readv, node in pack_readv_requests]
+            result.append((index, pack_readv, node_vector))
+        return total, result
+
     def _log_copied_texts(self):
         if 'pack' in debug.debug_flags:
             mutter('%s: create_pack: file texts copied: %s%s %d items t+%6.3fs',
@@ -886,6 +906,15 @@
             text_filter.extend([(fileid, file_revid) for file_revid in file_revids])
         self._text_filter = text_filter
 
+    def _revision_node_readv(self, revision_nodes):
+        """Return the total revisions and the readv's to issue.
+
+        :param revision_nodes: The revision index contents for the packs being
+            incorporated into the new pack.
+        :return: As per _least_readv_node_readv.
+        """
+        return self._least_readv_node_readv(revision_nodes)
+
     def _use_pack(self, new_pack):
         """Return True if new_pack should be used.
 
@@ -895,6 +924,40 @@
         return new_pack.data_inserted()
 
 
+class OptimisingPacker(Packer):
+    """A packer which spends more time to create better disk layouts."""
+
+    def _revision_node_readv(self, revision_nodes):
+        """Return the total revisions and the readv's to issue.
+
+        This sort places revisions in topological order with the ancestors
+        after the children.
+
+        :param revision_nodes: The revision index contents for the packs being
+            incorporated into the new pack.
+        :return: As per _least_readv_node_readv.
+        """
+        # build an ancestors dict
+        ancestors = {}
+        by_key = {}
+        for index, key, value, references in revision_nodes:
+            ancestors[key] = references[0]
+            by_key[key] = (index, value, references)
+        order = tsort.topo_sort(ancestors)
+        total = len(order)
+        # Single IO is pathological, but it will work as a starting point.
+        requests = []
+        for key in reversed(order):
+            index, value, references = by_key[key]
+            # ---- KnitGraphIndex.get_position
+            bits = value[1:].split(' ')
+            offset, length = int(bits[0]), int(bits[1])
+            requests.append(
+                (index, [(offset, length)], [(key, value[0], references)]))
+        # TODO: combine requests in the same index that are in ascending order.
+        return total, requests
+
+
 class ReconcilePacker(Packer):
     """A packer which regenerates indices etc as it copies.
     
@@ -968,8 +1031,9 @@
         del ideal_index
         del text_nodes
         # 3) bulk copy the ok data
-        list(self._copy_nodes_graph(ok_nodes, text_index_map,
-            self.new_pack._writer, self.new_pack.text_index))
+        total_items, readv_group_iter = self._least_readv_node_readv(ok_nodes)
+        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
+            self.new_pack.text_index, readv_group_iter, total_items))
         # 4) adhoc copy all the other texts.
         # We have to topologically insert all texts otherwise we can fail to
         # reconcile when parts of a single delta chain are preserved intact,
@@ -1164,17 +1228,18 @@
         self._execute_pack_operations(pack_operations)
         return True
 
-    def _execute_pack_operations(self, pack_operations):
+    def _execute_pack_operations(self, pack_operations, _packer_class=Packer):
         """Execute a series of pack operations.
 
         :param pack_operations: A list of [revision_count, packs_to_combine].
+        :param _packer_class: The class of packer to use (default: Packer).
         :return: None.
         """
         for revision_count, packs in pack_operations:
             # we may have no-ops from the setup logic
             if len(packs) == 0:
                 continue
-            Packer(self, packs, '.autopack').pack()
+            _packer_class(self, packs, '.autopack').pack()
             for pack in packs:
                 self._remove_pack_from_memory(pack)
         # record the newly available packs and stop advertising the old
@@ -1197,6 +1262,9 @@
         self.ensure_loaded()
         total_packs = len(self._names)
         if total_packs < 2:
+            # This is arguably wrong because we might not be optimal, but for
+            # now lets leave it in. (e.g. reconcile -> one pack. But not
+            # optimal.
             return
         total_revisions = self.revision_index.combined_index.key_count()
         # XXX: the following may want to be a class, to pack with a given
@@ -1208,10 +1276,9 @@
         pack_distribution = [1]
         pack_operations = [[0, []]]
         for pack in self.all_packs():
-            revision_count = pack.get_revision_count()
-            pack_operations[-1][0] += revision_count
+            pack_operations[-1][0] += pack.get_revision_count()
             pack_operations[-1][1].append(pack)
-        self._execute_pack_operations(pack_operations)
+        self._execute_pack_operations(pack_operations, OptimisingPacker)
 
     def plan_autopack_combinations(self, existing_packs, pack_distribution):
         """Plan a pack operation.

=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2007-11-29 23:22:01 +0000
+++ bzrlib/tests/HttpServer.py	2007-12-07 22:46:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -142,7 +142,8 @@
                                                                   file_size))
             self.end_headers()
             self.send_range_content(file, start, end - start + 1)
-            self.wfile.write("--%s\r\n" % boundary)
+        # Final boundary
+        self.wfile.write("--%s\r\n" % boundary)
 
     def do_GET(self):
         """Serve a GET request.

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2007-08-03 12:50:48 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2007-12-10 19:20:41 +0000
@@ -198,8 +198,6 @@
         child_tree.commit(message='child')
         self.check_revno(2)
 
-        self.run_bzr('bind', retcode=3)
-
     def test_commit_remote_bound(self):
         # It is not possible to commit to a branch
         # which is bound to a branch which is bound
@@ -247,12 +245,11 @@
         self.check_revno(2, 'base')
 
         os.chdir('child')
-        # These branches have diverged
-        self.run_bzr('bind ../base', retcode=3)
+        # These branches have diverged, but bind should succeed anyway
+        self.run_bzr('bind ../base')
 
-        # TODO: In the future, this might require actual changes
-        # to have occurred, rather than just a new revision entry
-        child_tree.merge_from_branch(base_branch)
+        # This should turn the local commit into a merge
+        child_tree.update()
         child_tree.commit(message='merged')
         self.check_revno(3)
 
@@ -261,16 +258,6 @@
         base_history = base_branch.revision_history()
         child_history = child_branch.revision_history()
 
-        # After a merge, trying to bind again should succeed
-        # keeping the new change as a local commit.
-        self.run_bzr('bind ../base')
-        self.check_revno(3)
-        self.check_revno(2, '../base')
-
-        # and compare the revision history now
-        self.assertEqual(base_history, base_branch.revision_history())
-        self.assertEqual(child_history, child_branch.revision_history())
-
     def test_bind_parent_ahead(self):
         base_tree = self.create_branches()[0]
 
@@ -322,6 +309,14 @@
         self.run_bzr('bind ../base')
         self.check_revno(1, '../base')
 
+    def test_bind_fail_if_missing(self):
+        """We should not be able to bind to a missing branch."""
+        tree = self.make_branch_and_tree('tree_1')
+        tree.commit('dummy commit')
+        self.run_bzr_error(['Not a branch.*no-such-branch/'], ['bind', '../no-such-branch'],
+                            working_dir='tree_1')
+        self.assertIs(None, tree.branch.get_bound_location())
+
     def test_commit_after_merge(self):
         base_tree, child_tree = self.create_branches()
 

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2007-11-30 02:06:35 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2007-12-10 16:39:00 +0000
@@ -520,11 +520,12 @@
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello.txt'])
         tree.add('hello.txt')
-        self.run_bzr("commit -m hello --author='John Doe <jdoe@example.com>' "
-                     "tree/hello.txt")
+        self.run_bzr(["commit", '-m', 'hello',
+                      '--author', u'John D\xf6 <jdoe@example.com>',
+                     "tree/hello.txt"])
         last_rev = tree.branch.repository.get_revision(tree.last_revision())
         properties = last_rev.properties
-        self.assertEqual('John Doe <jdoe@example.com>', properties['author'])
+        self.assertEqual(u'John D\xf6 <jdoe@example.com>', properties['author'])
 
     def test_author_no_email(self):
         """Author's name without an email address is allowed, too."""

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2007-11-26 13:55:51 +0000
+++ bzrlib/tests/blackbox/test_info.py	2007-12-15 18:33:10 +0000
@@ -1146,12 +1146,15 @@
             except errors.PathNotChild:
                 return path
 
-        if tree_locked and sys.platform == 'win32':
-            # We expect this to fail because of locking errors. (A write-locked
-            # file cannot be read-locked in the same process).
+        if tree_locked:
+            # We expect this to fail because of locking errors.
+            # (A write-locked file cannot be read-locked
+            # in the different process -- either on win32 or on linux).
             # This should be removed when the locking errors are fixed.
-            self.run_bzr_error([], 'info ' + command_string)
-            return
+            self.expectFailure('OS locks are exclusive '
+                'for different processes (Bug #174055)',
+                self.run_bzr_subprocess,
+                'info ' + command_string)
         out, err = self.run_bzr('info %s' % command_string)
         description = {
             (True, True): 'Lightweight checkout',

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2007-11-17 09:37:33 +0000
+++ bzrlib/tests/blackbox/test_log.py	2007-12-15 13:11:12 +0000
@@ -168,9 +168,16 @@
     def test_log_limit(self):
         self._prepare()
         log = self.run_bzr("log --limit 2")[0]
-        self.assertTrue('revno: 1\n' not in log)
-        self.assertTrue('revno: 2\n' in log)
-        self.assertTrue('revno: 3\n' in log)
+        self.assertNotContainsRe(log, r'revno: 1\n')
+        self.assertContainsRe(log, r'revno: 2\n')
+        self.assertContainsRe(log, r'revno: 3\n')
+
+    def test_log_limit_short(self):
+        self._prepare()
+        log = self.run_bzr("log -l 2")[0]
+        self.assertNotContainsRe(log, r'revno: 1\n')
+        self.assertContainsRe(log, r'revno: 2\n')
+        self.assertContainsRe(log, r'revno: 3\n')
 
 class TestLogMerges(ExternalBase):
 

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2007-11-28 17:27:10 +0000
+++ bzrlib/tests/blackbox/test_send.py	2007-11-30 21:10:18 +0000
@@ -17,6 +17,7 @@
 
 
 import os
+import sys
 from StringIO import StringIO
 
 from bzrlib import (
@@ -236,3 +237,12 @@
                                  'grandparent'])
         self.assertEqual('revision1', md.base_revision_id)
         self.assertEqual('revision3', md.revision_id)
+
+    def test_nonexistant_branch(self):
+        if sys.platform == "win32":
+            location = "C:/i/do/not/exist/"
+        else:
+            location = "/i/do/not/exist/"
+        out, err = self.run_bzr(["send", "--from", location], retcode=3)
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'bzr: ERROR: Not a branch: "%s".\n' % location)

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2007-11-01 19:53:57 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2007-12-11 14:36:31 +0000
@@ -227,3 +227,9 @@
         out, err = self.run_bzr('uncommit --force -r 2')
 
         self.assertEqual(['a2', 'b3', 'c3', 'c4', 'b4'], wt.get_parent_ids())
+
+    def test_uncommit_nonascii(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.commit(u'\u1234 message')
+        out, err = self.run_bzr('uncommit --force tree', encoding='ascii')
+        self.assertContainsRe(out, r'\? message')

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2007-12-07 05:46:51 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-12-10 19:20:41 +0000
@@ -28,6 +28,7 @@
     transactions,
     remote,
     repository,
+    tests,
     )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.delta import TreeDelta
@@ -638,7 +639,7 @@
         try:
             branch.bind(branch2)
         except errors.UpgradeRequired:
-            raise TestSkipped('Format does not support binding')
+            raise tests.TestNotApplicable('Format does not support binding')
         self.assertTrue(branch.unbind())
         self.assertFalse(branch.unbind())
         self.assertIs(None, branch.get_bound_location())
@@ -648,13 +649,25 @@
         try:
             self.assertIs(None, branch.get_old_bound_location())
         except errors.UpgradeRequired:
-            raise TestSkipped('Format does not store old bound locations')
+            raise tests.TestNotApplicable(
+                    'Format does not store old bound locations')
         branch2 = self.make_branch('branch2')
         branch.bind(branch2)
         self.assertIs(None, branch.get_old_bound_location())
         branch.unbind()
         self.assertContainsRe(branch.get_old_bound_location(), '\/branch2\/$')
 
+    def test_bind_diverged(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        tree_a.commit('rev1a')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        tree_a.commit('rev2a')
+        tree_b.commit('rev2b')
+        try:
+            tree_b.branch.bind(tree_a.branch)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+
 
 class TestStrict(TestCaseWithBranch):
 

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2007-12-04 18:11:51 +0000
+++ bzrlib/tests/test_errors.py	2007-12-07 23:03:34 +0000
@@ -56,6 +56,20 @@
             "The transport 'fpp' is only accessible within this process.",
             str(error))
 
+    def test_invalid_http_range(self):
+        error = errors.InvalidHttpRange('path',
+                                        'Content-Range: potatoes 0-00/o0oo0',
+                                        'bad range')
+        self.assertEquals("Invalid http range"
+                          " 'Content-Range: potatoes 0-00/o0oo0'"
+                          " for path: bad range",
+                          str(error))
+
+    def test_invalid_range(self):
+        error = errors.InvalidRange('path', 12, 'bad range')
+        self.assertEquals("Invalid range access in path at 12: bad range",
+                          str(error))
+
     def test_inventory_modified(self):
         error = errors.InventoryModified("a tree to be repred")
         self.assertEqualDiff("The current inventory for the tree 'a tree to "

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2007-11-30 16:48:24 +0000
+++ bzrlib/tests/test_http.py	2007-12-08 23:15:18 +0000
@@ -660,7 +660,22 @@
         self.assertEqual(l[2], (3, '34'))
         self.assertEqual(l[3], (9, '9'))
         # The server should have issued 4 requests
-        self.assertEqual(4, self.get_readonly_server().GET_request_nb)
+        self.assertEqual(4, server.GET_request_nb)
+
+    def test_readv_get_max_size(self):
+        server = self.get_readonly_server()
+        t = self._transport(server.get_url())
+        # force transport to issue multiple requests by limiting the number of
+        # bytes by request. Note that this apply to coalesced offsets only, a
+        # single range ill keep its size even if bigger than the limit.
+        t._get_max_size = 2
+        l = list(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
+        self.assertEqual(l[0], (0, '0'))
+        self.assertEqual(l[1], (1, '1'))
+        self.assertEqual(l[2], (2, '2345'))
+        self.assertEqual(l[3], (6, '6789'))
+        # The server should have issued 3 requests
+        self.assertEqual(3, server.GET_request_nb)
 
 
 class TestSingleRangeRequestServer(TestRangeRequestServer):
@@ -987,7 +1002,7 @@
     def _file_tail(self, relpath, tail_amount):
         code, data = self.transport._get(relpath, [], tail_amount)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
-        data.seek(-tail_amount + 1, 2)
+        data.seek(-tail_amount, 2)
         return data.read(tail_amount)
 
     def test_range_header(self):
@@ -997,10 +1012,10 @@
         # Tail
         self.assertEqual('789', self._file_tail('a', 3))
         # Syntactically invalid range
-        self.assertListRaises(errors.InvalidRange,
+        self.assertListRaises(errors.InvalidHttpRange,
                           self._file_contents, 'a', [(4, 3)])
         # Semantically invalid range
-        self.assertListRaises(errors.InvalidRange,
+        self.assertListRaises(errors.InvalidHttpRange,
                           self._file_contents, 'a', [(42, 128)])
 
 

=== modified file 'bzrlib/tests/test_http_response.py'
--- bzrlib/tests/test_http_response.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/test_http_response.py	2007-12-13 15:40:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,292 +14,414 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Tests from HTTP response parsing."""
+"""Tests from HTTP response parsing.
+
+The handle_response method read the response body of a GET request an returns
+the corresponding RangeFile.
+
+There are four different kinds of RangeFile:
+- a whole file whose size is unknown, seen as a simple byte stream,
+- a whole file whose size is known, we can't read past its end,
+- a single range file, a part of a file with a start and a size,
+- a multiple range file, several consecutive parts with known start offset
+  and size.
+
+Some properties are common to all kinds:
+- seek can only be forward (its really a socket underneath),
+- read can't cross ranges,
+- successive ranges are taken into account transparently,
+
+- the expected pattern of use is either seek(offset)+read(size) or a single
+  read with no size specified. For multiple range files, multiple read() will
+  return the corresponding ranges, trying to read further will raise
+  InvalidHttpResponse.
+"""
 
 from cStringIO import StringIO
-import mimetools
-
-from bzrlib import errors
-from bzrlib.transport import http
-from bzrlib.transport.http import response
-from bzrlib.tests import TestCase
-
-
-class TestResponseRange(TestCase):
-    """Test the ResponseRange class."""
-
-    def test_cmp(self):
-        RR = response.ResponseRange
-        r1 = RR(0, 10, 0)
-        r2 = RR(15, 20, 10)
-        self.assertTrue(r1 < r2)
-        self.assertFalse(r1 > r2)
-        self.assertTrue(r1 < 5)
-        self.assertFalse(r2 < 5)
-
-        self.assertEqual(RR(0, 10, 5), RR(0, 10, 5))
-        self.assertNotEqual(RR(0, 10, 5), RR(0, 8, 5))
-        self.assertNotEqual(RR(0, 10, 5), RR(0, 10, 6))
-
-    def test_sort_list(self):
-        """Ensure longer ranges are sorted after shorter ones"""
-        RR = response.ResponseRange
-        lst = [RR(3, 8, 0), 5, RR(3, 7, 0), 6]
-        lst.sort()
-        self.assertEqual([RR(3,7,0), RR(3,8,0), 5, 6], lst)
-
-
-class TestRangeFile(TestCase):
-    """Test RangeFile."""
-
-    def setUp(self):
-        content = "abcdefghijklmnopqrstuvwxyz"
-        self.fp = response.RangeFile('foo', StringIO(content))
-        self.fp._add_range(0,  9,   0)
-        self.fp._add_range(20, 29, 10)
-        self.fp._add_range(30, 39, 15)
-
-    def test_valid_accesses(self):
-        """Test so that valid accesses work to the file."""
-        self.fp.seek(0, 0)
-        self.assertEquals(self.fp.read(3), 'abc')
-        self.assertEquals(self.fp.read(3), 'def')
-        self.assertEquals(self.fp.tell(), 6)
-        self.fp.seek(20, 0)
-        self.assertEquals(self.fp.read(3), 'klm')
-        self.assertEquals(self.fp.read(2), 'no')
-        self.assertEquals(self.fp.tell(), 25)
-        # should wrap over to 30-39 entity
-        self.assertEquals(self.fp.read(3), 'pqr')
-        self.fp.seek(3)
-        self.assertEquals(self.fp.read(3), 'def')
-        self.assertEquals(self.fp.tell(), 6)
-
-    def test_invalid_accesses(self):
-        """Test so that invalid accesses trigger errors."""
-        self.fp.seek(9)
-        self.assertRaises(errors.InvalidRange, self.fp.read, 2)
-        self.fp.seek(39)
-        self.assertRaises(errors.InvalidRange, self.fp.read, 2)
-        self.fp.seek(19)
-        self.assertRaises(errors.InvalidRange, self.fp.read, 2)
-
-    def test__finish_ranges(self):
-        """Test that after RangeFile._finish_ranges the list is sorted."""
-        self.fp._add_range(1, 2, 3)
-        self.fp._add_range(8, 9, 10)
-        self.fp._add_range(3, 4, 5)
-
-        # TODO: jam 20060706 If we switch to inserting
-        #       in sorted order, remove this test
-        self.assertNotEqual(self.fp._ranges, sorted(self.fp._ranges))
-
-        self.fp._finish_ranges()
-        self.assertEqual(self.fp._ranges, sorted(self.fp._ranges))
-
-    def test_seek_and_tell(self):
-        # Check for seeking before start
-        self.fp.seek(-2, 0)
-        self.assertEqual(0, self.fp.tell())
-
-        self.fp.seek(5, 0)
-        self.assertEqual(5, self.fp.tell())
-
-        self.fp.seek(-2, 1)
-        self.assertEqual(3, self.fp.tell())
-
-        # TODO: jam 20060706 following tests will fail if this 
-        #       is not true, and would be difficult to debug
-        #       but it is a layering violation
-        self.assertEqual(39, self.fp._len)
-
-        self.fp.seek(0, 2)
-        self.assertEqual(39, self.fp.tell())
-
-        self.fp.seek(-10, 2)
-        self.assertEqual(29, self.fp.tell())
-
-        self.assertRaises(ValueError, self.fp.seek, 0, 4)
-        self.assertRaises(ValueError, self.fp.seek, 0, -1)
-
-
-class TestRegexes(TestCase):
-
-    def assertRegexMatches(self, groups, text):
-        """Check that the regex matches and returns the right values"""
-        m = self.regex.match(text)
-        self.assertNotEqual(None, m, "text %s did not match regex" % (text,))
-
-        self.assertEqual(groups, m.groups())
-
-    def test_range_re(self):
-        """Test that we match valid ranges."""
-        self.regex = response.HttpRangeResponse._CONTENT_RANGE_RE
-        self.assertRegexMatches(('bytes', '1', '10', '11'),
-                           'bytes 1-10/11')
-        self.assertRegexMatches(('bytes', '1', '10', '11'),
-                           '\tbytes  1-10/11   ')
-        self.assertRegexMatches(('bytes', '2123', '4242', '1231'),
-                           '\tbytes  2123-4242/1231   ')
-        self.assertRegexMatches(('chars', '1', '2', '3'),
-                           ' chars 1-2/3')
-
-    def test_content_type_re(self):
-        self.regex = response.HttpMultipartRangeResponse._CONTENT_TYPE_RE
-        self.assertRegexMatches(('', 'xxyyzz'),
-                                'multipart/byteranges; boundary = xxyyzz')
-        self.assertRegexMatches(('', 'xxyyzz'),
-                                'multipart/byteranges;boundary=xxyyzz')
-        self.assertRegexMatches(('', 'xx yy zz'),
-                                ' multipart/byteranges ; boundary= xx yy zz ')
-        self.assertRegexMatches(('"', 'xx yy zz'),
-                                ' multipart/byteranges ; boundary= "xx yy zz" ')
-        self.assertEqual(None,
-                         self.regex.match(
-                             ' multipart/byteranges ; boundary= "xx yy zz '))
-        self.assertEqual(None,
-                         self.regex.match(
-                             ' multipart/byteranges ; boundary= xx yy zz" '))
-        self.assertEqual(None,
-                self.regex.match('multipart byteranges;boundary=xx'))
-
-
-simple_data = """
---xxyyzz\r
-foo\r
-Content-range: bytes 1-10/20\r
-\r
-1234567890
---xxyyzz\r
-Content-Range: bytes 21-30/20\r
-bar\r
-\r
-abcdefghij
---xxyyzz\r
-content-range: bytes 41-50/20\r
-\r
-zyxwvutsrq
---xxyyzz\r
-content-range: bytes 51-60/20\r
-\r
-xxyyzz fbd
-"""
-
-
-class TestHelpers(TestCase):
-    """Test the helper functions"""
-
-    def test__parse_range(self):
-        """Test that _parse_range acts reasonably."""
-        content = StringIO('')
-        parse_range = response.HttpRangeResponse._parse_range
-        self.assertEqual((1,2), parse_range('bytes 1-2/3'))
-        self.assertEqual((10,20), parse_range('bytes 10-20/2'))
-
-        self.assertRaises(errors.InvalidHttpRange, parse_range, 'char 1-3/2')
-        self.assertRaises(errors.InvalidHttpRange, parse_range, 'bytes a-3/2')
-
-        try:
-            parse_range('bytes x-10/3', path='http://foo/bar')
-        except errors.InvalidHttpRange, e:
-            self.assertContainsRe(str(e), 'http://foo/bar')
-            self.assertContainsRe(str(e), 'bytes x-10/3')
-        else:
-            self.fail('Did not raise InvalidHttpRange')
-
-    def test__parse_boundary_simple(self):
-        """Test that _parse_boundary handles Content-type properly"""
-        parse_boundary = response.HttpMultipartRangeResponse._parse_boundary
-        m = parse_boundary(' multipart/byteranges; boundary=xxyyzz')
-        self.assertNotEqual(None, m)
-        # Check that the returned regex is capable of splitting simple_data
-        matches = list(m.finditer(simple_data))
-        self.assertEqual(4, len(matches))
-
-        # match.group() should be the content-range entry
-        # and match.end() should be the start of the content
-        self.assertEqual(' bytes 1-10/20', matches[0].group(1))
-        self.assertEqual(simple_data.find('1234567890'), matches[0].end())
-        self.assertEqual(' bytes 21-30/20', matches[1].group(1))
-        self.assertEqual(simple_data.find('abcdefghij'), matches[1].end())
-        self.assertEqual(' bytes 41-50/20', matches[2].group(1))
-        self.assertEqual(simple_data.find('zyxwvutsrq'), matches[2].end())
-        self.assertEqual(' bytes 51-60/20', matches[3].group(1))
-        self.assertEqual(simple_data.find('xxyyzz fbd'), matches[3].end())
-
-    def test__parse_boundary_invalid(self):
-        parse_boundary = response.HttpMultipartRangeResponse._parse_boundary
-        try:
-            parse_boundary(' multipart/bytes;boundary=xxyyzz',
-                           path='http://foo/bar')
-        except errors.InvalidHttpContentType, e:
-            self.assertContainsRe(str(e), 'http://foo/bar')
-            self.assertContainsRe(str(e), 'multipart/bytes;boundary=xxyyzz')
-        else:
-            self.fail('Did not raise InvalidHttpContentType')
-
-
-class TestHttpRangeResponse(TestCase):
-
-    def test_smoketest(self):
-        """A basic test that HttpRangeResponse is reasonable."""
-        content = StringIO('0123456789')
-        f = response.HttpRangeResponse('http://foo', 'bytes 1-10/9', content)
-        self.assertEqual([response.ResponseRange(1,10,0)], f._ranges)
-
+import httplib
+
+from bzrlib import (
+    errors,
+    tests,
+    )
+from bzrlib.transport.http import (
+    response,
+    _urllib2_wrappers,
+    )
+
+
+class ReadSocket(object):
+    """A socket-like object that can be given a predefined content."""
+
+    def __init__(self, data):
+        self.readfile = StringIO(data)
+
+    def makefile(self, mode='r', bufsize=None):
+        return self.readfile
+
+class FakeHTTPConnection(_urllib2_wrappers.HTTPConnection):
+
+    def __init__(self, sock):
+        _urllib2_wrappers.HTTPConnection.__init__(self, 'localhost')
+        # Set the socket to bypass the connection
+        self.sock = sock
+
+    def send(self, str):
+        """Ignores the writes on the socket."""
+        pass
+
+
+class TestHTTPConnection(tests.TestCase):
+
+    def test_cleanup_pipe(self):
+        sock = ReadSocket("""HTTP/1.1 200 OK\r
+Content-Type: text/plain; charset=UTF-8\r
+Content-Length: 18
+\r
+0123456789
+garbage""")
+        conn = FakeHTTPConnection(sock)
+        # Simulate the request sending so that the connection will be able to
+        # read the response.
+        conn.putrequest('GET', 'http://localhost/fictious')
+        conn.endheaders()
+        # Now, get the response
+        resp = conn.getresponse()
+        # Read part of the response
+        self.assertEquals('0123456789\n', resp.read(11))
+        # Override the thresold to force the warning emission
+        conn._range_warning_thresold = 6 # There are 7 bytes pending
+        conn.cleanup_pipe()
+        self.assertContainsRe(self._get_log(keep_log_file=True),
+                              'Got a 200 response when asking')
+
+
+class TestRangeFileMixin(object):
+    """Tests for accessing the first range in a RangeFile."""
+
+    # A simple string used to represent a file part (also called a range), in
+    # which offsets are easy to calculate for test writers. It's used as a
+    # building block with slight variations but basically 'a' is the first char
+    # of the range and 'z' is the last.
+    alpha = 'abcdefghijklmnopqrstuvwxyz'
+
+    def test_can_read_at_first_access(self):
+        """Test that the just created file can be read."""
+        self.assertEquals(self.alpha, self._file.read())
+
+    def test_seek_read(self):
+        """Test seek/read inside the range."""
+        f = self._file
+        start = self.first_range_start
+        # Before any use, tell() should be at the range start
+        self.assertEquals(start, f.tell())
+        cur = start # For an overall offset assertion
+        f.seek(start + 3)
+        cur += 3
+        self.assertEquals('def', f.read(3))
+        cur += len('def')
+        f.seek(4, 1)
+        cur += 4
+        self.assertEquals('klmn', f.read(4))
+        cur += len('klmn')
+        # read(0) in the middle of a range
+        self.assertEquals('', f.read(0))
+        # seek in place
+        here = f.tell()
+        f.seek(0, 1)
+        self.assertEquals(here, f.tell())
+        self.assertEquals(cur, f.tell())
+
+    def test_read_zero(self):
+        f = self._file
+        start = self.first_range_start
+        self.assertEquals('', f.read(0))
+        f.seek(10, 1)
+        self.assertEquals('', f.read(0))
+
+    def test_seek_at_range_end(self):
+        f = self._file
+        f.seek(26, 1)
+
+    def test_read_at_range_end(self):
+        """Test read behaviour at range end."""
+        f = self._file
+        self.assertEquals(self.alpha, f.read())
+        self.assertEquals('', f.read(0))
+        self.assertRaises(errors.InvalidRange, f.read, 1)
+
+    def test_unbounded_read_after_seek(self):
+        f = self._file
+        f.seek(24, 1)
+        # Should not cross ranges
+        self.assertEquals('yz', f.read())
+
+    def test_seek_backwards(self):
+        f = self._file
+        start = self.first_range_start
+        f.seek(start)
+        f.read(12)
+        self.assertRaises(errors.InvalidRange, f.seek, start + 5)
+
+    def test_seek_outside_single_range(self):
+        f = self._file
+        if f._size == -1 or f._boundary is not None:
+            raise tests.TestNotApplicable('Needs a fully defined range')
+        # Will seek past the range and then errors out
+        self.assertRaises(errors.InvalidRange,
+                          f.seek, self.first_range_start + 27)
+
+    def test_read_past_end_of_range(self):
+        f = self._file
+        if f._size == -1:
+            raise tests.TestNotApplicable("Can't check an unknown size")
+        start = self.first_range_start
+        f.seek(start + 20)
+        self.assertRaises(errors.InvalidRange, f.read, 10)
+
+    def test_seek_from_end(self):
+       """Test seeking from the end of the file.
+
+       The semantic is unclear in case of multiple ranges. Seeking from end
+       exists only for the http transports, cannot be used if the file size is
+       unknown and is not used in bzrlib itself. This test must be (and is)
+       overridden by daughter classes.
+
+       Reading from end makes sense only when a range has been requested from
+       the end of the file (see HttpTransportBase._get() when using the
+       'tail_amount' parameter). The HTTP response can only be a whole file or
+       a single range.
+       """
+       f = self._file
+       f.seek(-2, 2)
+       self.assertEquals('yz', f.read())
+
+
+class TestRangeFileSizeUnknown(tests.TestCase, TestRangeFileMixin):
+    """Test a RangeFile for a whole file whose size is not known."""
+
+    def setUp(self):
+        super(TestRangeFileSizeUnknown, self).setUp()
+        self._file = response.RangeFile('Whole_file_size_known',
+                                        StringIO(self.alpha))
+        # We define no range, relying on RangeFile to provide default values
+        self.first_range_start = 0 # It's the whole file
+
+    def test_seek_from_end(self):
+        """See TestRangeFileMixin.test_seek_from_end.
+
+        The end of the file can't be determined since the size is unknown.
+        """
+        self.assertRaises(errors.InvalidRange, self._file.seek, -1, 2)
+
+    def test_read_at_range_end(self):
+        """Test read behaviour at range end."""
+        f = self._file
+        self.assertEquals(self.alpha, f.read())
+        self.assertEquals('', f.read(0))
+        self.assertEquals('', f.read(1))
+
+class TestRangeFileSizeKnown(tests.TestCase, TestRangeFileMixin):
+    """Test a RangeFile for a whole file whose size is known."""
+
+    def setUp(self):
+        super(TestRangeFileSizeKnown, self).setUp()
+        self._file = response.RangeFile('Whole_file_size_known',
+                                        StringIO(self.alpha))
+        self._file.set_range(0, len(self.alpha))
+        self.first_range_start = 0 # It's the whole file
+
+
+class TestRangeFileSingleRange(tests.TestCase, TestRangeFileMixin):
+    """Test a RangeFile for a single range."""
+
+    def setUp(self):
+        super(TestRangeFileSingleRange, self).setUp()
+        self._file = response.RangeFile('Single_range_file',
+                                        StringIO(self.alpha))
+        self.first_range_start = 15
+        self._file.set_range(self.first_range_start, len(self.alpha))
+
+
+    def test_read_before_range(self):
+        # This can't occur under normal circumstances, we have to force it
+        f = self._file
+        f._pos = 0 # Force an invalid pos
+        self.assertRaises(errors.InvalidRange, f.read, 2)
+
+class TestRangeFilMultipleRanges(tests.TestCase, TestRangeFileMixin):
+    """Test a RangeFile for multiple ranges.
+
+    The RangeFile used for the tests contains three ranges:
+
+    - at offset 25: alpha
+    - at offset 100: alpha
+    - at offset 126: alpha.upper()
+
+    The two last ranges are contiguous. This only rarely occurs (should not in
+    fact) in real uses but may lead to hard to track bugs.
+    """
+
+    def setUp(self):
+        super(TestRangeFilMultipleRanges, self).setUp()
+
+        boundary = 'separation'
+
+        content = ''
+        self.first_range_start = 25
+        file_size = 200 # big enough to encompass all ranges
+        for (start, part) in [(self.first_range_start, self.alpha),
+                              # Two contiguous ranges
+                              (100, self.alpha),
+                              (126, self.alpha.upper())]:
+            content += self._multipart_byterange(part, start, boundary,
+                                                 file_size)
+        # Final boundary
+        content += self._boundary_line(boundary)
+
+        self._file = response.RangeFile('Multiple_ranges_file',
+                                        StringIO(content))
+        # Ranges are set by decoding the range headers, the RangeFile user is
+        # supposed to call the following before using seek or read since it
+        # requires knowing the *response* headers (in that case the boundary
+        # which is part of the Content-Type header).
+        self._file.set_boundary(boundary)
+
+    def _boundary_line(self, boundary):
+        """Helper to build the formatted boundary line."""
+        return '--' + boundary + '\r\n'
+
+    def _multipart_byterange(self, data, offset, boundary, file_size='*'):
+        """Encode a part of a file as a multipart/byterange MIME type.
+
+        When a range request is issued, the HTTP response body can be
+        decomposed in parts, each one representing a range (start, size) in a
+        file.
+
+        :param data: The payload.
+        :param offset: where data starts in the file
+        :param boundary: used to separate the parts
+        :param file_size: the size of the file containing the range (default to
+            '*' meaning unknown)
+
+        :return: a string containing the data encoded as it will appear in the
+            HTTP response body.
+        """
+        bline = self._boundary_line(boundary)
+        # Each range begins with a boundary line
+        range = bline
+        # A range is described by a set of headers, but only 'Content-Range' is
+        # required for our implementation (TestHandleResponse below will
+        # exercise ranges with multiple or missing headers')
+        range += 'Content-Range: bytes %d-%d/%d\r\n' % (offset,
+                                                        offset+len(data)-1,
+                                                        file_size)
+        range += '\r\n'
+        # Finally the raw bytes
+        range += data
+        return range
+
+    def test_read_all_ranges(self):
+        f = self._file
+        self.assertEquals(self.alpha, f.read()) # Read first range
+        f.seek(100) # Trigger the second range recognition
+        self.assertEquals(self.alpha, f.read()) # Read second range
+        self.assertEquals(126, f.tell())
+        f.seek(126) # Start of third range which is also the current pos !
+        self.assertEquals('A', f.read(1))
+        f.seek(10, 1)
+        self.assertEquals('LMN', f.read(3))
+
+    def test_seek_from_end(self):
+        """See TestRangeFileMixin.test_seek_from_end."""
+        # The actual implementation will seek from end for the first range only
+        # and then fail. Since seeking from end is intended to be used for a
+        # single range only anyway, this test just document the actual
+        # behaviour.
+        f = self._file
+        f.seek(-2, 2)
+        self.assertEquals('yz', f.read())
+        self.assertRaises(errors.InvalidRange, f.seek, -2, 2)
+
+    def test_seek_into_void(self):
+        f = self._file
+        start = self.first_range_start
+        f.seek(start)
+        # Seeking to a point between two ranges is possible (only once) but
+        # reading there is forbidden
+        f.seek(start + 40)
+        # We crossed a range boundary, so now the file is positioned at the
+        # start of the new range (i.e. trying to seek below 100 will error out)
+        f.seek(100)
+        f.seek(125)
+
+    def test_seek_across_ranges(self):
+        f = self._file
+        start = self.first_range_start
+        f.seek(126) # skip the two first ranges
+        self.assertEquals('AB', f.read(2))
+
+    def test_seek_twice_between_ranges(self):
+        f = self._file
+        start = self.first_range_start
+        f.seek(start + 40) # Past the first range but before the second
+        # Now the file is positioned at the second range start (100)
+        self.assertRaises(errors.InvalidRange, f.seek, start + 41)
+
+    def test_seek_at_range_end(self):
+        """Test seek behavior at range end."""
+        f = self._file
+        f.seek(25 + 25)
+        f.seek(100 + 25)
+        f.seek(126 + 25)
+
+    def test_read_at_range_end(self):
+        f = self._file
+        self.assertEquals(self.alpha, f.read())
+        self.assertEquals(self.alpha, f.read())
+        self.assertEquals(self.alpha.upper(), f.read())
+        self.assertRaises(errors.InvalidHttpResponse, f.read, 1)
+
+
+class TestRangeFileVarious(tests.TestCase):
+    """Tests RangeFile aspects not covered elsewhere."""
+
+    def test_seek_whence(self):
+        """Test the seek whence parameter values."""
+        f = response.RangeFile('foo', StringIO('abc'))
+        f.set_range(0, 3)
         f.seek(0)
-        self.assertRaises(errors.InvalidRange, f.read, 2)
-        f.seek(1)
-        self.assertEqual('012345', f.read(6))
-
-    def test_invalid(self):
-        try:
-            f = response.HttpRangeResponse('http://foo', 'bytes x-10/9',
-                                           StringIO('0123456789'))
-        except errors.InvalidHttpRange, e:
-            self.assertContainsRe(str(e), 'http://foo')
-            self.assertContainsRe(str(e), 'bytes x-10/9')
-        else:
-            self.fail('Failed to raise InvalidHttpRange')
-
-
-class TestHttpMultipartRangeResponse(TestCase):
-    """Test the handling of multipart range responses"""
-
-    def test_simple(self):
-        content = StringIO(simple_data)
-        multi = response.HttpMultipartRangeResponse('http://foo',
-                    'multipart/byteranges; boundary = xxyyzz', content)
-
-        self.assertEqual(4, len(multi._ranges))
-
-        multi.seek(1)
-        self.assertEqual('1234567890', multi.read(10))
-        multi.seek(21)
-        self.assertEqual('abcdefghij', multi.read(10))
-        multi.seek(41)
-        self.assertEqual('zyxwvutsrq', multi.read(10))
-        multi.seek(51)
-        self.assertEqual('xxyyzz fbd', multi.read(10))
-        # TODO: jam 20060706 Currently RangeFile does not support
-        #       reading across ranges. Consider adding it.
-        multi.seek(41)
-        # self.assertEqual('zyxwvutsrqxxyyzz fbd', multi.read(20))
-        self.assertRaises(errors.InvalidRange, multi.read, 20)
-
-        multi.seek(21)
-        self.assertRaises(errors.InvalidRange, multi.read, 11)
-        multi.seek(31)
-        self.assertRaises(errors.InvalidRange, multi.read, 10)
-
-    def test_invalid(self):
-        content = StringIO('')
-        try:
-            response.HttpMultipartRangeResponse('http://foo',
-                        'multipart/byte;boundary=invalid', content)
-        except errors.InvalidHttpContentType, e:
-            self.assertContainsRe(str(e), 'http://foo')
-            self.assertContainsRe(str(e), 'multipart/byte;')
+        f.seek(1, 1)
+        f.seek(-1, 2)
+        self.assertRaises(ValueError, f.seek, 0, 14)
+
+    def test_range_syntax(self):
+        """Test the Content-Range scanning."""
+
+        f = response.RangeFile('foo', StringIO())
+
+        def ok(expected, header_value):
+            f.set_range_from_header(header_value)
+            # Slightly peek under the covers to get the size
+            self.assertEquals(expected, (f.tell(), f._size))
+
+        ok((1, 10), 'bytes 1-10/11')
+        ok((1, 10), 'bytes 1-10/*')
+        ok((12, 2), '\tbytes 12-13/*')
+        ok((28, 1), '  bytes 28-28/*')
+        ok((2123, 2120), 'bytes  2123-4242/12310')
+        ok((1, 10), 'bytes 1-10/ttt') # We don't check total (ttt)
+
+        def nok(header_value):
+            self.assertRaises(errors.InvalidHttpRange,
+                              f.set_range_from_header, header_value)
+
+        nok('bytes 10-2/3')
+        nok('chars 1-2/3')
+        nok('bytes xx-yyy/zzz')
+        nok('bytes xx-12/zzz')
+        nok('bytes 11-yy/zzz')
+        nok('bytes10-2/3')
 
 
 # Taken from real request responses
@@ -317,25 +439,6 @@
 """)
 
 
-_missing_response = (404, """HTTP/1.1 404 Not Found\r
-Date: Tue, 11 Jul 2006 04:32:56 GMT\r
-Server: Apache/2.0.54 (Fedora)\r
-Content-Length: 336\r
-Connection: close\r
-Content-Type: text/html; charset=iso-8859-1\r
-\r
-""", """<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
-<html><head>
-<title>404 Not Found</title>
-</head><body>
-<h1>Not Found</h1>
-<p>The requested URL /branches/bzr/jam-integration/.bzr/repository/format was not found on this server.</p>
-<hr>
-<address>Apache/2.0.54 (Fedora) Server at bzr.arbash-meinel.com Port 80</address>
-</body></html>
-""")
-
-
 _single_range_response = (206, """HTTP/1.1 206 Partial Content\r
 Date: Tue, 11 Jul 2006 04:45:22 GMT\r
 Server: Apache/2.0.54 (Fedora)\r
@@ -414,6 +517,7 @@
 --418470f848b63279b--\r
 """)
 
+
 _multipart_squid_range_response = (206, """HTTP/1.0 206 Partial Content\r
 Date: Thu, 31 Aug 2006 21:16:22 GMT\r
 Server: Apache/2.2.2 (Unix) DAV/2\r
@@ -446,34 +550,60 @@
 """)
 
 
-_redirect_response = (206, """HTTP/1.1 301 Moved Permanently\r
-Date: Tue, 18 Jul 2006 20:29:22 GMT\r
-Server: Apache/2.0.54 (Ubuntu) PHP/4.4.0-3ubuntu1 mod_ssl/2.0.54 OpenSSL/0.9.7g\r
-Location: http://bazaar-vcs.org/bzr/bzr.dev/.bzr/repository/inventory.knit\r
-Content-Length: 272\r
-Keep-Alive: timeout=15, max=100\r
-Connection: Keep-Alive\r
-Content-Type: text/html; charset=iso-8859-1\r
-\r
-HTTP/1.1 206 Partial Content\r
-Date: Tue, 18 Jul 2006 20:29:23 GMT\r
-Server: Apache/2.0.54 (Ubuntu) PHP/4.4.0-3ubuntu1 mod_ssl/2.0.54 OpenSSL/0.9.7g\r
-Last-Modified: Tue, 18 Jul 2006 20:24:59 GMT\r
-ETag: "be8213-83958c-f0d3dcc0"\r
-Accept-Ranges: bytes\r
-Content-Length: 425\r
-Content-Range: bytes 8623075-8623499/8623500\r
-Keep-Alive: timeout=15, max=100\r
-Connection: Keep-Alive\r
-Content-Type: text/plain; charset=UTF-8\r
-\r
-""", """this data intentionally removed, 
-this is not meant to be tested by
-handle_response, just _extract_headers
-""")
-
-
 # This is made up
+_full_text_response_no_content_type = (200, """HTTP/1.1 200 OK\r
+Date: Tue, 11 Jul 2006 04:32:56 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Sun, 23 Apr 2006 19:35:20 GMT\r
+ETag: "56691-23-38e9ae00"\r
+Accept-Ranges: bytes\r
+Content-Length: 35\r
+Connection: close\r
+\r
+""", """Bazaar-NG meta directory, format 1
+""")
+
+
+_full_text_response_no_content_length = (200, """HTTP/1.1 200 OK\r
+Date: Tue, 11 Jul 2006 04:32:56 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Sun, 23 Apr 2006 19:35:20 GMT\r
+ETag: "56691-23-38e9ae00"\r
+Accept-Ranges: bytes\r
+Connection: close\r
+Content-Type: text/plain; charset=UTF-8\r
+\r
+""", """Bazaar-NG meta directory, format 1
+""")
+
+
+_single_range_no_content_range = (206, """HTTP/1.1 206 Partial Content\r
+Date: Tue, 11 Jul 2006 04:45:22 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Thu, 06 Jul 2006 20:22:05 GMT\r
+ETag: "238a3c-16ec2-805c5540"\r
+Accept-Ranges: bytes\r
+Content-Length: 100\r
+Connection: close\r
+\r
+""", """mbp@sourcefrog.net-20050309040815-13242001617e4a06
+mbp@sourcefrog.net-20050309040929-eee0eb3e6d1e762""")
+
+
+_single_range_response_truncated = (206, """HTTP/1.1 206 Partial Content\r
+Date: Tue, 11 Jul 2006 04:45:22 GMT\r
+Server: Apache/2.0.54 (Fedora)\r
+Last-Modified: Thu, 06 Jul 2006 20:22:05 GMT\r
+ETag: "238a3c-16ec2-805c5540"\r
+Accept-Ranges: bytes\r
+Content-Length: 100\r
+Content-Range: bytes 100-199/93890\r
+Connection: close\r
+Content-Type: text/plain; charset=UTF-8\r
+\r
+""", """mbp@sourcefrog.net-20050309040815-13242001617e4a06""")
+
+
 _invalid_response = (444, """HTTP/1.1 444 Bad Response\r
 Date: Tue, 11 Jul 2006 04:32:56 GMT\r
 Connection: close\r
@@ -490,113 +620,51 @@
 """)
 
 
-# This should be in test_http.py, but the headers we
-# want to parse are here
-class TestExtractHeader(TestCase):
-    
-    def use_response(self, response):
-        self.headers = http._extract_headers(response[1], 'http://foo')
-
-    def check_header(self, header, value):
-        self.assertEqual(value, self.headers[header])
-        
-    def test_full_text(self):
-        self.use_response(_full_text_response)
-
-        self.check_header('Date', 'Tue, 11 Jul 2006 04:32:56 GMT')
-        self.check_header('date', 'Tue, 11 Jul 2006 04:32:56 GMT')
-        self.check_header('Content-Length', '35')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-        self.check_header('content-type', 'text/plain; charset=UTF-8')
-
-    def test_missing_response(self):
-        self.use_response(_missing_response)
-
-        self.check_header('Content-Length', '336')
-        self.check_header('Content-Type', 'text/html; charset=iso-8859-1')
-
-    def test_single_range(self):
-        self.use_response(_single_range_response)
-
-        self.check_header('Content-Length', '100')
-        self.check_header('Content-Range', 'bytes 100-199/93890')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-
-    def test_single_range_no_content(self):
-        self.use_response(_single_range_no_content_type)
-
-        self.check_header('Content-Length', '100')
-        self.check_header('Content-Range', 'bytes 100-199/93890')
-
-    def test_multi_range(self):
-        self.use_response(_multipart_range_response)
-
-        self.check_header('Content-Length', '1534')
-        self.check_header('Content-Type',
-                          'multipart/byteranges; boundary=418470f848b63279b')
-
-    def test_multi_squid_range(self):
-        self.use_response(_multipart_squid_range_response)
-
-        self.check_header('Content-Length', '598')
-        self.check_header('Content-Type',
-                          'multipart/byteranges; '\
-                          'boundary="squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196"')
-
-    def test_redirect(self):
-        """We default to returning the last group of headers in the file."""
-        self.use_response(_redirect_response)
-        self.check_header('Content-Range', 'bytes 8623075-8623499/8623500')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-
-    def test_empty(self):
-        self.assertRaises(errors.InvalidHttpResponse,
-            http._extract_headers, '', 'bad url')
-
-    def test_no_opening_http(self):
-        # Remove the HTTP line from the header
-        first, txt = _full_text_response[1].split('\r\n', 1)
-        self.assertRaises(errors.InvalidHttpResponse,
-            http._extract_headers, txt, 'missing HTTTP')
-
-    def test_trailing_whitespace(self):
-        # Test that we ignore bogus whitespace on the end
-        code, txt, body = _full_text_response
-        txt += '\r\n\n\n\n\n'
-        self.use_response((code, txt, body))
-
-        self.check_header('Date', 'Tue, 11 Jul 2006 04:32:56 GMT')
-        self.check_header('Content-Length', '35')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-
-    def test_trailing_non_http(self):
-        # Test that we ignore bogus stuff on the end
-        code, txt, body = _full_text_response
-        txt = txt + 'Foo: Bar\r\nBaz: Bling\r\n\r\n'
-        self.use_response((code, txt, body))
-
-        self.check_header('Date', 'Tue, 11 Jul 2006 04:32:56 GMT')
-        self.check_header('Content-Length', '35')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-        self.assertRaises(KeyError, self.headers.__getitem__, 'Foo')
-
-    def test_extra_whitespace(self):
-        # Test that we read an HTTP response, even with extra whitespace
-        code, txt, body = _redirect_response
-        # Find the second HTTP location
-        loc = txt.find('HTTP', 5)
-        txt = txt[:loc] + '\r\n\n' + txt[loc:]
-        self.use_response((code, txt, body))
-        self.check_header('Content-Range', 'bytes 8623075-8623499/8623500')
-        self.check_header('Content-Type', 'text/plain; charset=UTF-8')
-
-
-class TestHandleResponse(TestCase):
-    
+_multipart_no_content_range = (206, """HTTP/1.0 206 Partial Content\r
+Content-Type: multipart/byteranges; boundary=THIS_SEPARATES\r
+Content-Length: 598\r
+\r
+""",
+"""\r
+--THIS_SEPARATES\r
+Content-Type: text/plain\r
+\r
+# bzr knit index 8
+--THIS_SEPARATES\r
+""")
+
+
+_multipart_no_boundary = (206, """HTTP/1.0 206 Partial Content\r
+Content-Type: multipart/byteranges; boundary=THIS_SEPARATES\r
+Content-Length: 598\r
+\r
+""",
+"""\r
+--THIS_SEPARATES\r
+Content-Type: text/plain\r
+Content-Range: bytes 0-18/18672\r
+\r
+# bzr knit index 8
+
+The range ended at the line above, this text is garbage instead of a boundary
+line
+""")
+
+
+class TestHandleResponse(tests.TestCase):
+
+    def _build_HTTPMessage(self, raw_headers):
+        status_and_headers = StringIO(raw_headers)
+        # Get rid of the status line
+        status_and_headers.readline()
+        msg = httplib.HTTPMessage(status_and_headers)
+        return msg
+
     def get_response(self, a_response):
         """Process a supplied response, and return the result."""
-        headers = http._extract_headers(a_response[1], 'http://foo')
-        return response.handle_response('http://foo', a_response[0], headers,
+        code, raw_headers, body = a_response
+        msg = self._build_HTTPMessage(raw_headers)
+        return response.handle_response('http://foo', code, msg,
                                         StringIO(a_response[2]))
 
     def test_full_text(self):
@@ -604,31 +672,25 @@
         # It is a StringIO from the original data
         self.assertEqual(_full_text_response[2], out.read())
 
-    def test_missing_response(self):
-        self.assertRaises(errors.NoSuchFile,
-            self.get_response, _missing_response)
-
     def test_single_range(self):
         out = self.get_response(_single_range_response)
-        self.assertIsInstance(out, response.HttpRangeResponse)
-
-        self.assertRaises(errors.InvalidRange, out.read, 20)
 
         out.seek(100)
         self.assertEqual(_single_range_response[2], out.read(100))
 
     def test_single_range_no_content(self):
         out = self.get_response(_single_range_no_content_type)
-        self.assertIsInstance(out, response.HttpRangeResponse)
-
-        self.assertRaises(errors.InvalidRange, out.read, 20)
 
         out.seek(100)
         self.assertEqual(_single_range_no_content_type[2], out.read(100))
 
+    def test_single_range_truncated(self):
+        out = self.get_response(_single_range_response_truncated)
+        # Content-Range declares 100 but only 51 present
+        self.assertRaises(errors.ShortReadvError, out.seek, out.tell() + 51)
+
     def test_multi_range(self):
         out = self.get_response(_multipart_range_response)
-        self.assertIsInstance(out, response.HttpMultipartRangeResponse)
 
         # Just make sure we can read the right contents
         out.seek(0)
@@ -639,7 +701,6 @@
 
     def test_multi_squid_range(self):
         out = self.get_response(_multipart_squid_range_response)
-        self.assertIsInstance(out, response.HttpMultipartRangeResponse)
 
         # Just make sure we can read the right contents
         out.seek(0)
@@ -650,30 +711,37 @@
 
     def test_invalid_response(self):
         self.assertRaises(errors.InvalidHttpResponse,
-            self.get_response, _invalid_response)
+                          self.get_response, _invalid_response)
 
     def test_full_text_no_content_type(self):
         # We should not require Content-Type for a full response
-        a_response = _full_text_response
-        headers = http._extract_headers(a_response[1], 'http://foo')
-        del headers['Content-Type']
-        out = response.handle_response('http://foo', a_response[0], headers,
-                                        StringIO(a_response[2]))
-        self.assertEqual(_full_text_response[2], out.read())
+        code, raw_headers, body = _full_text_response_no_content_type
+        msg = self._build_HTTPMessage(raw_headers)
+        out = response.handle_response('http://foo', code, msg, StringIO(body))
+        self.assertEqual(body, out.read())
 
-    def test_missing_no_content_type(self):
-        # Without Content-Type we should still raise NoSuchFile on a 404
-        a_response = _missing_response
-        headers = http._extract_headers(a_response[1], 'http://missing')
-        del headers['Content-Type']
-        self.assertRaises(errors.NoSuchFile,
-            response.handle_response, 'http://missing', a_response[0], headers,
-                                      StringIO(a_response[2]))
+    def test_full_text_no_content_length(self):
+        code, raw_headers, body = _full_text_response_no_content_length
+        msg = self._build_HTTPMessage(raw_headers)
+        out = response.handle_response('http://foo', code, msg, StringIO(body))
+        self.assertEqual(body, out.read())
 
     def test_missing_content_range(self):
-        a_response = _single_range_response
-        headers = http._extract_headers(a_response[1], 'http://nocontent')
-        del headers['Content-Range']
-        self.assertRaises(errors.InvalidHttpResponse,
-            response.handle_response, 'http://nocontent', a_response[0],
-                                      headers, StringIO(a_response[2]))
+        code, raw_headers, body = _single_range_no_content_range
+        msg = self._build_HTTPMessage(raw_headers)
+        self.assertRaises(errors.InvalidHttpResponse,
+                          response.handle_response,
+                          'http://bogus', code, msg, StringIO(body))
+
+    def test_multipart_no_content_range(self):
+        code, raw_headers, body = _multipart_no_content_range
+        msg = self._build_HTTPMessage(raw_headers)
+        self.assertRaises(errors.InvalidHttpResponse,
+                          response.handle_response,
+                          'http://bogus', code, msg, StringIO(body))
+
+    def test_multipart_no_boundary(self):
+        out = self.get_response(_multipart_no_boundary)
+        out.read()  # Read the whole range
+        # Fail to find the boundary line
+        self.assertRaises(errors.InvalidHttpResponse, out.seek, 1, 1)

=== modified file 'bzrlib/tests/test_merge_directive.py'
--- bzrlib/tests/test_merge_directive.py	2007-11-22 00:05:12 +0000
+++ bzrlib/tests/test_merge_directive.py	2007-12-10 15:20:39 +0000
@@ -553,6 +553,15 @@
         revision = md.install_revisions(tree_b.branch.repository)
         self.assertEqual('rev2a', revision)
 
+    def test_use_submit_for_missing_dependency(self):
+        tree_a, tree_b, branch_c = self.make_trees()
+        branch_c.pull(tree_a.branch)
+        self.build_tree_contents([('tree_a/file', 'content_q\ncontent_r\n')])
+        tree_a.commit('rev3a', rev_id='rev3a')
+        md = self.from_objects(tree_a.branch.repository, 'rev3a', 500, 36,
+            branch_c.base, base_revision_id='rev2a')
+        revision = md.install_revisions(tree_b.branch.repository)
+
 
 class TestMergeDirective1Branch(tests.TestCaseWithTransport,
     TestMergeDirectiveBranch):
@@ -564,7 +573,10 @@
 
     def from_objects(self, repository, revision_id, time, timezone,
         target_branch, patch_type='bundle', local_target_branch=None,
-        public_branch=None, message=None):
+        public_branch=None, message=None, base_revision_id=None):
+        if base_revision_id is not None:
+            raise tests.TestNotApplicable('This format does not support'
+                                          ' explicit bases.')
         repository.lock_write()
         try:
             return merge_directive.MergeDirective.from_objects( repository,

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2007-11-29 17:32:23 +0000
+++ bzrlib/tests/test_reconfigure.py	2007-12-13 16:38:54 +0000
@@ -18,6 +18,7 @@
     branch as _mod_branch,
     errors,
     reconfigure,
+    repository,
     tests,
     workingtree,
     )
@@ -49,8 +50,8 @@
 
     def test_repo_to_branch(self):
         repo = self.make_repository('repo')
-        self.assertRaises(errors.ReconfigurationNotSupported,
-                          reconfigure.Reconfigure.to_branch, repo.bzrdir)
+        reconfiguration = reconfigure.Reconfigure.to_branch(repo.bzrdir)
+        reconfiguration.apply()
 
     def test_checkout_to_branch(self):
         branch = self.make_branch('branch')
@@ -146,6 +147,24 @@
                                                               parent.base)
         reconfiguration.apply()
 
+    def test_tree_to_lightweight_checkout(self):
+        # A tree with no related branches and no supplied bind location cannot
+        # become a checkout
+        parent = self.make_branch('parent')
+
+        tree = self.make_branch_and_tree('tree')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            tree.bzrdir)
+        self.assertRaises(errors.NoBindLocation, reconfiguration.apply)
+        # setting a parent allows it to become a checkout
+        tree.branch.set_parent(parent.base)
+        reconfiguration.apply()
+        # supplying a location allows it to become a checkout
+        tree2 = self.make_branch_and_tree('tree2')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            tree2.bzrdir, parent.base)
+        reconfiguration.apply()
+
     def test_checkout_to_checkout(self):
         parent = self.make_branch('parent')
         checkout = parent.create_checkout('checkout')
@@ -187,3 +206,31 @@
         self.assertRaises(errors.AlreadyLightweightCheckout,
                           reconfigure.Reconfigure.to_lightweight_checkout,
                           checkout.bzrdir)
+
+    def test_repo_to_tree(self):
+        repo = self.make_repository('repo')
+        reconfiguration = reconfigure.Reconfigure.to_tree(repo.bzrdir)
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+
+    def test_shared_repo_to_lightweight_checkout(self):
+        repo = self.make_repository('repo', shared=True)
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir)
+        self.assertRaises(errors.NoBindLocation, reconfiguration.apply)
+        branch = self.make_branch('branch')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir, 'branch')
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+        repository.Repository.open('repo')
+
+    def test_unshared_repo_to_lightweight_checkout(self):
+        repo = self.make_repository('repo', shared=False)
+        branch = self.make_branch('branch')
+        reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
+            repo.bzrdir, 'branch')
+        reconfiguration.apply()
+        workingtree.WorkingTree.open('repo')
+        self.assertRaises(errors.NoRepositoryPresent,
+                          repository.Repository.open, 'repo')

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2007-11-30 15:59:02 +0000
+++ bzrlib/tests/test_repository.py	2007-12-04 02:19:39 +0000
@@ -929,6 +929,26 @@
         self.assertEqual(1, len(list(index.iter_all_entries())))
         self.assertEqual(2, len(tree.branch.repository.all_revision_ids()))
 
+    def test_pack_layout(self):
+        format = self.get_format()
+        tree = self.make_branch_and_tree('.', format=format)
+        trans = tree.branch.repository.bzrdir.get_repository_transport(None)
+        tree.commit('start', rev_id='1')
+        tree.commit('more work', rev_id='2')
+        tree.branch.repository.pack()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        pack = tree.branch.repository._pack_collection.get_pack_by_name(
+            tree.branch.repository._pack_collection.names()[0])
+        # revision access tends to be tip->ancestor, so ordering that way on 
+        # disk is a good idea.
+        for _1, key, val, refs in pack.revision_index.iter_all_entries():
+            if key == ('1',):
+                pos_1 = int(val[1:].split()[0])
+            else:
+                pos_2 = int(val[1:].split()[0])
+        self.assertTrue(pos_2 < pos_1)
+
     def test_pack_repositories_support_multiple_write_locks(self):
         format = self.get_format()
         self.make_repository('.', shared=True, format=format)

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2007-11-30 15:48:27 +0000
+++ bzrlib/tests/test_revision.py	2007-12-10 05:03:31 +0000
@@ -134,6 +134,7 @@
         sources = br1
 
         br1.lock_read()
+        br2.lock_read()
         self.addCleanup(br1.unlock)
         br2.lock_read()
         self.addCleanup(br2.unlock)

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2007-12-02 02:29:02 +0000
+++ bzrlib/tests/test_transform.py	2007-12-11 19:21:27 +0000
@@ -1124,6 +1124,24 @@
         self.assertContainsRe(str(err),
             "^File exists: .+/foo")
 
+    def test_two_directories_clash_finalize(self):
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = TreeTransform(wt)  # TreeTransform obtains write lock
+            try:
+                foo_1 = tt.new_directory('foo', tt.root)
+                tt.new_directory('bar', foo_1)
+                foo_2 = tt.new_directory('foo', tt.root)
+                tt.new_directory('baz', foo_2)
+                # Lie to tt that we've already resolved all conflicts.
+                tt.apply(no_conflicts=True)
+            except:
+                tt.finalize()
+                raise
+        err = self.assertRaises(errors.FileExists, tt_helper)
+        self.assertContainsRe(str(err),
+            "^File exists: .+/foo")
+
 
 class TransformGroup(object):
 

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2007-11-19 13:44:25 +0000
+++ bzrlib/tests/test_transport.py	2007-12-10 10:41:24 +0000
@@ -152,11 +152,12 @@
 
 
 class TestCoalesceOffsets(TestCase):
-    
-    def check(self, expected, offsets, limit=0, fudge=0):
+
+    def check(self, expected, offsets, limit=0, max_size=0, fudge=0):
         coalesce = Transport._coalesce_offsets
         exp = [_CoalescedOffset(*x) for x in expected]
-        out = list(coalesce(offsets, limit=limit, fudge_factor=fudge))
+        out = list(coalesce(offsets, limit=limit, fudge_factor=fudge,
+                            max_size=max_size))
         self.assertEqual(exp, out)
 
     def test_coalesce_empty(self):
@@ -169,7 +170,7 @@
         self.check([(0, 10, [(0, 10)]),
                     (20, 10, [(0, 10)]),
                    ], [(0, 10), (20, 10)])
-            
+
     def test_coalesce_unsorted(self):
         self.check([(20, 10, [(0, 10)]),
                     (0, 10, [(0, 10)]),
@@ -179,6 +180,7 @@
         self.check([(0, 20, [(0, 10), (10, 10)])],
                    [(0, 10), (10, 10)])
 
+    # XXX: scary, http.readv() can't handle that --vila20071209
     def test_coalesce_overlapped(self):
         self.check([(0, 15, [(0, 10), (5, 10)])],
                    [(0, 10), (5, 10)])
@@ -208,6 +210,20 @@
                    ], [(10, 10), (30, 10), (100, 10)],
                    fudge=10
                   )
+    def test_coalesce_max_size(self):
+        self.check([(10, 20, [(0, 10), (10, 10)]),
+                    (30, 50, [(0, 50)]),
+                    # If one range is above max_size, it gets its own coalesced
+                    # offset
+                    (100, 80, [(0, 80),]),],
+                   [(10, 10), (20, 10), (30, 50), (100, 80)],
+                   max_size=50
+                  )
+
+    def test_coalesce_no_max_size(self):
+        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)]),],
+                   [(10, 10), (20, 10), (30, 50), (80, 100)],
+                  )
 
 
 class TestMemoryTransport(TestCase):

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2007-11-25 14:42:29 +0000
+++ bzrlib/tests/test_transport_implementations.py	2007-12-10 10:41:24 +0000
@@ -20,6 +20,7 @@
 TransportTestProviderAdapter.
 """
 
+import itertools
 import os
 from cStringIO import StringIO
 from StringIO import StringIO as pyStringIO
@@ -186,11 +187,15 @@
         self.build_tree(files, transport=t, line_endings='binary')
         self.check_transport_contents('contents of a\n', t, 'a')
         content_f = t.get_multi(files)
-        for content, f in zip(contents, content_f):
+        # Use itertools.izip() instead of use zip() or map(), since they fully
+        # evaluate their inputs, the transport requests should be issued and
+        # handled sequentially (we don't want to force transport to buffer).
+        for content, f in itertools.izip(contents, content_f):
             self.assertEqual(content, f.read())
 
         content_f = t.get_multi(iter(files))
-        for content, f in zip(contents, content_f):
+        # Use itertools.izip() for the same reason
+        for content, f in itertools.izip(contents, content_f):
             self.assertEqual(content, f.read())
 
         self.assertRaises(NoSuchFile, t.get, 'c')
@@ -978,7 +983,7 @@
         self.check_transport_contents('c this file\n', t, 'b')
 
         # TODO: Try to write a test for atomicity
-        # TODO: Test moving into a non-existant subdirectory
+        # TODO: Test moving into a non-existent subdirectory
         # TODO: Test Transport.move_multi
 
     def test_copy(self):
@@ -1281,7 +1286,7 @@
         self.assertEqual('', t.relpath(t.base))
         # base ends with /
         self.assertEqual('', t.relpath(t.base[:-1]))
-        # subdirs which dont exist should still give relpaths.
+        # subdirs which don't exist should still give relpaths.
         self.assertEqual('foo', t.relpath(t.base + 'foo'))
         # trailing slash should be the same.
         self.assertEqual('foo', t.relpath(t.base + 'foo/'))
@@ -1569,7 +1574,7 @@
                 adjust_for_latency=True, upper_limit=content_size))
             self.assertEqual(1, len(result))
             data_len = len(result[0][1])
-            # minimmum length is from 400 to 1034 - 634
+            # minimum length is from 400 to 1034 - 634
             self.assertTrue(data_len >= 634)
             # must contain the region 400 to 1034
             self.assertTrue(result[0][0] <= 400)

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2007-12-02 02:29:02 +0000
+++ bzrlib/transform.py	2007-12-11 19:21:27 +0000
@@ -997,6 +997,7 @@
         new_paths = self.new_paths()
         modified_paths = []
         child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        completed_new = []
         try:
             for num, (path, trans_id) in enumerate(new_paths):
                 new_entry = None
@@ -1019,7 +1020,7 @@
                             self.rename_count += 1
                     if trans_id in self._new_contents:
                         modified_paths.append(full_path)
-                        del self._new_contents[trans_id]
+                        completed_new.append(trans_id)
 
                 if trans_id in self._new_id:
                     if kind is None:
@@ -1064,6 +1065,8 @@
                                             new_entry))
         finally:
             child_pb.finished()
+        for trans_id in completed_new:
+            del self._new_contents[trans_id]
         return modified_paths
 
     def _set_executability(self, path, entry, trans_id):

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2007-11-27 08:26:00 +0000
+++ bzrlib/transport/__init__.py	2007-12-08 23:15:18 +0000
@@ -234,6 +234,10 @@
         return cmp((self.start, self.length, self.ranges),
                    (other.start, other.length, other.ranges))
 
+    def __repr__(self):
+        return '%s(%r, %r, %r)' % (self.__class__.__name__,
+            self.start, self.length, self.ranges)
+
 
 class LateReadError(object):
     """A helper for transports which pretends to be a readable file.
@@ -788,7 +792,7 @@
         return offsets
 
     @staticmethod
-    def _coalesce_offsets(offsets, limit, fudge_factor):
+    def _coalesce_offsets(offsets, limit=0, fudge_factor=0, max_size=0):
         """Yield coalesced offsets.
 
         With a long list of neighboring requests, combine them
@@ -797,13 +801,21 @@
         Turns  [(15, 10), (25, 10)] => [(15, 20, [(0, 10), (10, 10)])]
 
         :param offsets: A list of (start, length) pairs
-        :param limit: Only combine a maximum of this many pairs
-                      Some transports penalize multiple reads more than
-                      others, and sometimes it is better to return early.
-                      0 means no limit
+
+        :param limit: Only combine a maximum of this many pairs Some transports
+                penalize multiple reads more than others, and sometimes it is
+                better to return early.
+                0 means no limit
+
         :param fudge_factor: All transports have some level of 'it is
                 better to read some more data and throw it away rather 
                 than seek', so collapse if we are 'close enough'
+
+        :param max_size: Create coalesced offsets no bigger than this size.
+                When a single offset is bigger than 'max_size', it will keep
+                its size and be alone in the coalesced offset.
+                0 means no maximum size.
+
         :return: yield _CoalescedOffset objects, which have members for where
                 to start, how much to read, and how to split those 
                 chunks back up
@@ -813,10 +825,11 @@
 
         for start, size in offsets:
             end = start + size
-            if (last_end is not None 
+            if (last_end is not None
                 and start <= last_end + fudge_factor
                 and start >= cur.start
-                and (limit <= 0 or len(cur.ranges) < limit)):
+                and (limit <= 0 or len(cur.ranges) < limit)
+                and (max_size <= 0 or end - cur.start <= max_size)):
                 cur.length = end - cur.start
                 cur.ranges.append((start-cur.start, size))
             else:

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2007-11-30 09:51:22 +0000
+++ bzrlib/transport/http/__init__.py	2007-12-12 21:06:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,49 +78,6 @@
     return url
 
 
-def _extract_headers(header_text, url):
-    """Extract the mapping for an rfc2822 header
-
-    This is a helper function for the test suite and for _pycurl.
-    (urllib already parses the headers for us)
-
-    In the case that there are multiple headers inside the file,
-    the last one is returned.
-
-    :param header_text: A string of header information.
-        This expects that the first line of a header will always be HTTP ...
-    :param url: The url we are parsing, so we can raise nice errors
-    :return: mimetools.Message object, which basically acts like a case 
-        insensitive dictionary.
-    """
-    first_header = True
-    remaining = header_text
-
-    if not remaining:
-        raise errors.InvalidHttpResponse(url, 'Empty headers')
-
-    while remaining:
-        header_file = StringIO(remaining)
-        first_line = header_file.readline()
-        if not first_line.startswith('HTTP'):
-            if first_header: # The first header *must* start with HTTP
-                raise errors.InvalidHttpResponse(url,
-                    'Opening header line did not start with HTTP: %s'
-                    % (first_line,))
-            else:
-                break # We are done parsing
-        first_header = False
-        m = mimetools.Message(header_file)
-
-        # mimetools.Message parses the first header up to a blank line
-        # So while there is remaining data, it probably means there is
-        # another header to be parsed.
-        # Get rid of any preceeding whitespace, which if it is all whitespace
-        # will get rid of everything.
-        remaining = header_file.read().lstrip()
-    return m
-
-
 class HttpTransportBase(ConnectedTransport, medium.SmartClientMedium):
     """Base class for http implementations.
 
@@ -175,7 +132,12 @@
         :param relpath: The relative path to the file
         """
         code, response_file = self._get(relpath, None)
-        return response_file
+        # FIXME: some callers want an iterable... One step forward, three steps
+        # backwards :-/ And not only an iterable, but an iterable that can be
+        # seeked backwards, so we will never be able to do that.  One such
+        # known client is bzrlib.bundle.serializer.v4.get_bundle_reader. At the
+        # time of this writing it's even the only known client -- vila20071203
+        return StringIO(response_file.read())
 
     def _get(self, relpath, ranges, tail_amount=0):
         """Get a file, or part of a file.
@@ -213,30 +175,6 @@
             # further tries were unsuccessful
             raise exc_info[0], exc_info[1], exc_info[2]
 
-    def _get_ranges_hinted(self, relpath, ranges):
-        """Issue a ranged GET request taking server capabilities into account.
-
-        Depending of the errors returned by the server, we try several GET
-        requests, trying to minimize the data transferred.
-
-        :param relpath: Path relative to transport base URL
-        :param ranges: None to get the whole file;
-            or  a list of _CoalescedOffset to fetch parts of a file.
-        :returns: A file handle containing at least the requested ranges.
-        """
-        exc_info = None
-        try_again = True
-        while try_again:
-            try_again = False
-            try:
-                code, f = self._get(relpath, ranges)
-            except errors.InvalidRange, e:
-                if exc_info is None:
-                    exc_info = sys.exc_info()
-                self._degrade_range_hint(relpath, ranges, exc_info)
-                try_again = True
-        return f
-
     # _coalesce_offsets is a helper for readv, it try to combine ranges without
     # degrading readv performances. _bytes_to_read_before_seek is the value
     # used for the limit parameter and has been tuned for other transports. For
@@ -254,6 +192,9 @@
     # By default Apache has a limit of ~400 ranges before replying with a 400
     # Bad Request. So we go underneath that amount to be safe.
     _max_get_ranges = 200
+    # We impose no limit on the range size. But see _pycurl.py for a different
+    # use.
+    _get_max_size = 0
 
     def _readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
@@ -262,7 +203,7 @@
         :param return: A list or generator of (offset, data) tuples
         """
 
-        # offsets may be a genarator, we will iterate it several times, so
+        # offsets may be a generator, we will iterate it several times, so
         # build a list
         offsets = list(offsets)
 
@@ -274,7 +215,8 @@
             sorted_offsets = sorted(offsets)
             coalesced = self._coalesce_offsets(
                 sorted_offsets, limit=self._max_readv_combine,
-                fudge_factor=self._bytes_to_read_before_seek)
+                fudge_factor=self._bytes_to_read_before_seek,
+                max_size=self._get_max_size)
 
             # Turn it into a list, we will iterate it several times
             coalesced = list(coalesced)
@@ -284,23 +226,30 @@
             # Cache the data read, but only until it's been used
             data_map = {}
             # We will iterate on the data received from the GET requests and
-            # serve the corresponding offsets repecting the initial order. We
+            # serve the corresponding offsets respecting the initial order. We
             # need an offset iterator for that.
             iter_offsets = iter(offsets)
             cur_offset_and_size = iter_offsets.next()
 
             try:
-                for cur_coal, file in self._coalesce_readv(relpath, coalesced):
+                for cur_coal, rfile in self._coalesce_readv(relpath, coalesced):
                     # Split the received chunk
                     for offset, size in cur_coal.ranges:
                         start = cur_coal.start + offset
-                        file.seek(start, 0)
-                        data = file.read(size)
+                        rfile.seek(start, 0)
+                        data = rfile.read(size)
                         data_len = len(data)
                         if data_len != size:
                             raise errors.ShortReadvError(relpath, start, size,
                                                          actual=data_len)
-                        data_map[(start, size)] = data
+                        if (start, size) == cur_offset_and_size:
+                            # The offset requested are sorted as the coalesced
+                            # ones, no need to cache. Win !
+                            yield cur_offset_and_size[0], data
+                            cur_offset_and_size = iter_offsets.next()
+                        else:
+                            # Different sorting. We need to cache.
+                            data_map[(start, size)] = data
 
                     # Yield everything we can
                     while cur_offset_and_size in data_map:
@@ -311,7 +260,8 @@
                         yield cur_offset_and_size[0], this_data
                         cur_offset_and_size = iter_offsets.next()
 
-            except (errors.ShortReadvError,errors.InvalidRange), e:
+            except (errors.ShortReadvError, errors.InvalidRange,
+                    errors.InvalidHttpRange), e:
                 self._degrade_range_hint(relpath, coalesced, sys.exc_info())
                 # Some offsets may have been already processed, so we retry
                 # only the unsuccessful ones.
@@ -320,25 +270,52 @@
 
     def _coalesce_readv(self, relpath, coalesced):
         """Issue several GET requests to satisfy the coalesced offsets"""
-        total = len(coalesced)
-        if self._range_hint == 'multi':
-             max_ranges = self._max_get_ranges
-        elif self._range_hint == 'single':
-             max_ranges = total
+
+        def get_and_yield(relpath, coalesced):
+            if coalesced:
+                # Note that the _get below may raise
+                # errors.InvalidHttpRange. It's the caller's responsibility to
+                # decide how to retry since it may provide different coalesced
+                # offsets.
+                code, rfile = self._get(relpath, coalesced)
+                for coal in coalesced:
+                    yield coal, rfile
+
+        if self._range_hint is None:
+            # Download whole file
+            for c, rfile in get_and_yield(relpath, coalesced):
+                yield c, rfile
         else:
-            # The whole file will be downloaded anyway
-            max_ranges = total
-        # TODO: Some web servers may ignore the range requests and return the
-        # whole file, we may want to detect that and avoid further requests.
-        # Hint: test_readv_multiple_get_requests will fail in that case .
-        for group in xrange(0, len(coalesced), max_ranges):
-            ranges = coalesced[group:group+max_ranges]
-            # Note that the following may raise errors.InvalidRange. It's the
-            # caller responsability to decide how to retry since it may provide
-            # different coalesced offsets.
-            code, file = self._get(relpath, ranges)
-            for range in ranges:
-                yield range, file
+            total = len(coalesced)
+            if self._range_hint == 'multi':
+                max_ranges = self._max_get_ranges
+            elif self._range_hint == 'single':
+                max_ranges = total
+            else:
+                raise AssertionError("Unknown _range_hint %r"
+                                     % (self._range_hint,))
+            # TODO: Some web servers may ignore the range requests and return
+            # the whole file, we may want to detect that and avoid further
+            # requests.
+            # Hint: test_readv_multiple_get_requests will fail once we do that
+            cumul = 0
+            ranges = []
+            for coal in coalesced:
+                if ((self._get_max_size > 0
+                     and cumul + coal.length > self._get_max_size)
+                    or len(ranges) >= max_ranges):
+                    # Get that much and yield
+                    for c, rfile in get_and_yield(relpath, ranges):
+                        yield c, rfile
+                    # Restart with the current offset
+                    ranges = [coal]
+                    cumul = coal.length
+                else:
+                    ranges.append(coal)
+                    cumul += coal.length
+            # Get the rest and yield
+            for c, rfile in get_and_yield(relpath, ranges):
+                yield c, rfile
 
     def recommended_page_size(self):
         """See Transport.recommended_page_size().
@@ -348,34 +325,6 @@
         """
         return 64 * 1024
 
-    @staticmethod
-    @deprecated_method(zero_seventeen)
-    def offsets_to_ranges(offsets):
-        """Turn a list of offsets and sizes into a list of byte ranges.
-
-        :param offsets: A list of tuples of (start, size).  An empty list
-            is not accepted.
-        :return: a list of inclusive byte ranges (start, end) 
-            Adjacent ranges will be combined.
-        """
-        # Make sure we process sorted offsets
-        offsets = sorted(offsets)
-
-        prev_end = None
-        combined = []
-
-        for start, size in offsets:
-            end = start + size - 1
-            if prev_end is None:
-                combined.append([start, end])
-            elif start <= prev_end + 1:
-                combined[-1][1] = end
-            else:
-                combined.append([start, end])
-            prev_end = end
-
-        return combined
-
     def _post(self, body_bytes):
         """POST body_bytes to .bzr/smart on this transport.
         
@@ -490,7 +439,11 @@
             return self.__class__(self.abspath(offset), self)
 
     def _attempted_range_header(self, offsets, tail_amount):
-        """Prepare a HTTP Range header at a level the server should accept"""
+        """Prepare a HTTP Range header at a level the server should accept.
+
+        :return: the range header representing offsets/tail_amount or None if
+            no header can be built.
+        """
 
         if self._range_hint == 'multi':
             # Generate the header describing all offsets

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2007-11-30 09:51:22 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-12-08 23:15:18 +0000
@@ -33,6 +33,7 @@
 
 import os
 from cStringIO import StringIO
+import httplib
 import sys
 
 from bzrlib import (
@@ -45,7 +46,6 @@
 from bzrlib.trace import mutter
 from bzrlib.transport.http import (
     ca_bundle,
-    _extract_headers,
     HttpTransportBase,
     response,
     )
@@ -208,6 +208,14 @@
 
         return code, data
 
+    # The parent class use 0 to minimize the requests, but since we can't
+    # exploit the results as soon as they are received (pycurl limitation) we'd
+    # better issue more requests and provide a more responsive UI do the cost
+    # of more latency costs.
+    # If you modify this, think about modifying the comment in http/__init__.py
+    # too.
+    _get_max_size = 4 * 1024 * 1024
+
     def _get_ranged(self, relpath, offsets, tail_amount):
         """Make a request for just part of the file."""
         curl = self._get_curl()
@@ -222,10 +230,25 @@
         data.seek(0)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
-        # mutter('header:\n%r', header.getvalue())
-        headers = _extract_headers(header.getvalue(), abspath)
-        # handle_response will raise NoSuchFile, etc based on the response code
-        return code, response.handle_response(abspath, code, headers, data)
+
+        if code == 404: # not found
+            raise errors.NoSuchFile(abspath)
+        elif code in (400, 416):
+            # We don't know which, but one of the ranges we specified was
+            # wrong.
+            raise errors.InvalidHttpRange(abspath, range_header,
+                                          'Server return code %d'
+                                          % curl.getinfo(pycurl.HTTP_CODE))
+        msg = self._parse_headers(header)
+        return code, response.handle_response(abspath, code, msg, data)
+
+    def _parse_headers(self, status_and_headers):
+        """Transform the headers provided by curl into an HTTPMessage"""
+        status_and_headers.seek(0)
+        # Ignore status line
+        status_and_headers.readline()
+        msg = httplib.HTTPMessage(status_and_headers)
+        return msg
 
     def _post(self, body_bytes):
         fake_file = StringIO(body_bytes)
@@ -242,8 +265,8 @@
         self._curl_perform(curl, header, ['Expect: '])
         data.seek(0)
         code = curl.getinfo(pycurl.HTTP_CODE)
-        headers = _extract_headers(header.getvalue(), abspath)
-        return code, response.handle_response(abspath, code, headers, data)
+        msg = self._parse_headers(header)
+        return code, response.handle_response(abspath, code, msg, data)
 
     def _raise_curl_http_error(self, curl, info=None):
         code = curl.getinfo(pycurl.HTTP_CODE)
@@ -297,21 +320,19 @@
                 raise errors.ConnectionError(
                     'curl connection error (%s)\non %s' % (e[1], url))
             elif e[0] == CURLE_PARTIAL_FILE:
-                # Pycurl itself has detected a short read.  We do
-                # not have all the information for the
-                # ShortReadvError, but that should be enough
+                # Pycurl itself has detected a short read.  We do not have all
+                # the information for the ShortReadvError, but that should be
+                # enough
                 raise errors.ShortReadvError(url,
                                              offset='unknown', length='unknown',
                                              actual='unknown',
                                              extra='Server aborted the request')
-            # jam 20060713 The code didn't use to re-raise the exception here,
-            # but that seemed bogus
             raise
         code = curl.getinfo(pycurl.HTTP_CODE)
         if code in (301, 302, 303, 307):
             url = curl.getinfo(pycurl.EFFECTIVE_URL)
-            headers = _extract_headers(header.getvalue(), url)
-            redirected_to = headers['Location']
+            msg = self._parse_headers(header)
+            redirected_to = msg.getheader('location')
             raise errors.RedirectRequested(url,
                                            redirected_to,
                                            is_permanent=(code == 301),

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2007-11-30 09:51:22 +0000
+++ bzrlib/transport/http/_urllib.py	2007-12-03 08:33:06 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
     urlutils,
     )
 from bzrlib.transport import http
-# TODO: handle_response should be integrated into the _urllib2_wrappers
+# TODO: handle_response should be integrated into the http/__init__.py
 from bzrlib.transport.http.response import handle_response
 from bzrlib.transport.http._urllib2_wrappers import (
     Opener,
@@ -70,6 +70,9 @@
             # Give back shared info
             request.connection = connection
             (auth, proxy_auth) = self._get_credentials()
+            # Clean the httplib.HTTPConnection pipeline in case the previous
+            # request couldn't do it
+            connection.cleanup_pipe()
         else:
             # First request, intialize credentials.
             # scheme and realm will be set by the _urllib2_wrappers.AuthHandler
@@ -123,6 +126,7 @@
             if range_header is not None:
                 accepted_errors.append(206)
                 accepted_errors.append(400)
+                accepted_errors.append(416)
                 bytes = 'bytes=' + range_header
                 headers = {'Range': bytes}
 
@@ -132,21 +136,21 @@
 
         code = response.code
         if code == 404: # not found
-            self._get_connection().fake_close()
             raise errors.NoSuchFile(abspath)
+        elif code in (400, 416):
+            # We don't know which, but one of the ranges we specified was
+            # wrong.
+            raise errors.InvalidHttpRange(abspath, range_header,
+                                          'Server return code %d' % code)
 
-        data = handle_response(abspath, code, response.headers, response)
-        # Close response to free the httplib.HTTPConnection pipeline
-        self._get_connection().fake_close()
+        data = handle_response(abspath, code, response.info(), response)
         return code, data
 
     def _post(self, body_bytes):
         abspath = self._remote_path('.bzr/smart')
         response = self._perform(Request('POST', abspath, body_bytes))
         code = response.code
-        data = handle_response(abspath, code, response.headers, response)
-        # Close response to free the httplib.HTTPConnection pipeline
-        self._get_connection().fake_close()
+        data = handle_response(abspath, code, response.info(), response)
         return code, data
 
     def _head(self, relpath):
@@ -159,7 +163,6 @@
                           accepted_errors=[200, 404])
         response = self._perform(request)
 
-        self._get_connection().fake_close()
         return response
 
     def has(self, relpath):

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2007-11-30 09:51:22 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2007-12-13 15:44:44 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006,2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 For instance, we create a new HTTPConnection and HTTPSConnection that inherit
 from the original urllib2.HTTP(s)Connection objects, but also have a new base
-which implements a custom getresponse and fake_close handlers.
+which implements a custom getresponse and cleanup_pipe handlers.
 
 And then we implement custom HTTPHandler and HTTPSHandler classes, that use
 the custom HTTPConnection classes.
@@ -78,9 +78,6 @@
     # Some responses have bodies in which we have no interest
     _body_ignored_responses = [301,302, 303, 307, 401, 403, 404]
 
-    def __init__(self, *args, **kwargs):
-        httplib.HTTPResponse.__init__(self, *args, **kwargs)
-
     def begin(self):
         """Begin to read the response from the server.
 
@@ -123,6 +120,41 @@
             # below we keep the socket with the server opened.
             self.will_close = False
 
+    # in finish() below, we may have to discard several MB in the worst
+    # case. To avoid buffering that much, we read and discard by chunks
+    # instead. The underlying file is either a socket or a StringIO, so reading
+    # 8k chunks should be fine.
+    _discarded_buf_size = 8192
+
+    def finish(self):
+        """Finish reading the body.
+
+        In some cases, the client may have left some bytes to read in the
+        body. That will block the next request to succeed if we use a
+        persistent connection. If we don't use a persistent connection, well,
+        nothing will block the next request since a new connection will be
+        issued anyway.
+
+        :return: the number of bytes left on the socket (may be None)
+        """
+        pending = None
+        if not self.isclosed():
+            # Make sure nothing was left to be read on the socket
+            pending = 0
+            while self.length and self.length > self._discarded_buf_size:
+                data = self.read(self._discarded_buf_size)
+                pending += len(data)
+            if self.length:
+                data = self.read(self.length)
+                pending += len(data)
+            if pending:
+                trace.mutter(
+                    "bogus http server didn't give body length,"
+                    "%s bytes left on the socket",
+                    pending)
+            self.close()
+        return pending
+
 
 # Not inheriting from 'object' because httplib.HTTPConnection doesn't.
 class AbstractHTTPConnection:
@@ -131,11 +163,40 @@
     response_class = Response
     strict = 1 # We don't support HTTP/0.9
 
-    def fake_close(self):
-        """Make the connection believes the response have been fully handled.
-
-        That makes the httplib.HTTPConnection happy
-        """
+    # When we detect a server responding with the whole file to range requests,
+    # we want to warn. But not below a given thresold.
+    _range_warning_thresold = 1024 * 1024
+
+    def __init__(self):
+        self._response = None
+        self._ranges_received_whole_file = None
+
+    def _mutter_connect(self):
+        netloc = '%s:%s' % (self.host, self.port)
+        if self.proxied_host is not None:
+            netloc += '(proxy for %s)' % self.proxied_host
+        trace.mutter('* About to connect() to %s' % netloc)
+
+    def getresponse(self):
+        """Capture the response to be able to cleanup"""
+        self._response = httplib.HTTPConnection.getresponse(self)
+        return self._response
+
+    def cleanup_pipe(self):
+        """Make the connection believe the response has been fully processed."""
+        if self._response is not None:
+            pending = self._response.finish()
+            # Warn the user (once)
+            if (self._ranges_received_whole_file is None
+                and self._response.status == 200
+                and pending and pending > self._range_warning_thresold
+                ):
+                self._ranges_received_whole_file = True
+                trace.warning(
+                    'Got a 200 response when asking for multiple ranges,'
+                    ' does your server at %s:%s support range requests?',
+                    self.host, self.port)
+            self._response = None
         # Preserve our preciousss
         sock = self.sock
         self.sock = None
@@ -149,26 +210,28 @@
 
     # XXX: Needs refactoring at the caller level.
     def __init__(self, host, port=None, strict=None, proxied_host=None):
-        if 'http' in debug.debug_flags:
-            netloc = host
-            if port is not None:
-                netloc += '%d' % port
-            if proxied_host is not None:
-                netloc += '(proxy for %s)' % proxied_host
-            trace.mutter('* About to connect() to %s' % netloc)
+        AbstractHTTPConnection.__init__(self)
         httplib.HTTPConnection.__init__(self, host, port, strict)
         self.proxied_host = proxied_host
 
+    def connect(self):
+        if 'http' in debug.debug_flags:
+            self._mutter_connect()
+        httplib.HTTPConnection.connect(self)
+
 
 class HTTPSConnection(AbstractHTTPConnection, httplib.HTTPSConnection):
 
     def __init__(self, host, port=None, key_file=None, cert_file=None,
                  strict=None, proxied_host=None):
+        AbstractHTTPConnection.__init__(self)
         httplib.HTTPSConnection.__init__(self, host, port,
                                          key_file, cert_file, strict)
         self.proxied_host = proxied_host
 
     def connect(self):
+        if 'http' in debug.debug_flags:
+            self._mutter_connect()
         httplib.HTTPConnection.connect(self)
         if self.proxied_host is None:
             self.connect_to_origin()
@@ -385,7 +448,6 @@
                     print '  Will retry, %s %r' % (method, url)
                 request.connection.close()
                 response = self.do_open(http_class, request, False)
-                convert_to_addinfourl = False
             else:
                 if self._debuglevel > 0:
                     print 'Received second exception: [%r]' % exc_val
@@ -421,7 +483,7 @@
                     print '  Failed again, %s %r' % (method, url)
                     print '  Will raise: [%r]' % my_exception
                 raise my_exception, None, exc_tb
-        return response, convert_to_addinfourl
+        return response
 
     def do_open(self, http_class, request, first_try=True):
         """See urllib2.AbstractHTTPHandler.do_open for the general idea.
@@ -455,9 +517,8 @@
             convert_to_addinfourl = True
         except (socket.gaierror, httplib.BadStatusLine, httplib.UnknownProtocol,
                 socket.error, httplib.HTTPException):
-            response, convert_to_addinfourl = self.retry_or_raise(http_class,
-                                                                  request,
-                                                                  first_try)
+            response = self.retry_or_raise(http_class, request, first_try)
+            convert_to_addinfourl = False
 
 # FIXME: HTTPConnection does not fully support 100-continue (the
 # server responses are just ignored)
@@ -471,17 +532,6 @@
 #            connection.send(body)
 #            response = connection.getresponse()
 
-        if 'http' in debug.debug_flags:
-            version = 'HTTP/%d.%d'
-            try:
-                version = version % (response.version / 10,
-                                     response.version % 10)
-            except:
-                version = 'HTTP/%r' % version
-            trace.mutter('< %s %s %s' % (version, response.status,
-                                            response.reason))
-            hdrs = [h.rstrip('\r\n') for h in response.msg.headers]
-            trace.mutter('< ' + '\n< '.join(hdrs) + '\n')
         if self._debuglevel > 0:
             print 'Receives response: %r' % response
             print '  For: %r(%r)' % (request.get_method(),
@@ -496,37 +546,28 @@
             resp = urllib2.addinfourl(fp, r.msg, req.get_full_url())
             resp.code = r.status
             resp.msg = r.reason
+            resp.version = r.version
             if self._debuglevel > 0:
                 print 'Create addinfourl: %r' % resp
                 print '  For: %r(%r)' % (request.get_method(),
                                          request.get_full_url())
+            if 'http' in debug.debug_flags:
+                version = 'HTTP/%d.%d'
+                try:
+                    version = version % (resp.version / 10,
+                                         resp.version % 10)
+                except:
+                    version = 'HTTP/%r' % resp.version
+                trace.mutter('< %s %s %s' % (version, resp.code,
+                                             resp.msg))
+                # Use the raw header lines instead of treating resp.info() as a
+                # dict since we may miss duplicated headers otherwise.
+                hdrs = [h.rstrip('\r\n') for h in resp.info().headers]
+                trace.mutter('< ' + '\n< '.join(hdrs) + '\n')
         else:
             resp = response
         return resp
 
-#       # we need titled headers in a dict but
-#       # response.getheaders returns a list of (lower(header).
-#       # Let's title that because most of bzr handle titled
-#       # headers, but maybe we should switch to lowercased
-#       # headers...
-#        # jam 20060908: I think we actually expect the headers to
-#        #       be similar to mimetools.Message object, which uses
-#        #       case insensitive keys. It lowers() all requests.
-#        #       My concern is that the code may not do perfect title case.
-#        #       For example, it may use Content-type rather than Content-Type
-#
-#        # When we get rid of addinfourl, we must ensure that bzr
-#        # always use titled headers and that any header received
-#        # from server is also titled.
-#
-#        headers = {}
-#        for header, value in (response.getheaders()):
-#            headers[header.title()] = value
-#        # FIXME: Implements a secured .read method
-#        response.code = response.status
-#        response.headers = headers
-#        return response
-
 
 class HTTPHandler(AbstractHTTPHandler):
     """A custom handler that just thunks into HTTPConnection"""
@@ -566,7 +607,7 @@
                 raise ConnectionError("Can't connect to %s via proxy %s" % (
                         connect.proxied_host, self.host))
             # Housekeeping
-            connection.fake_close()
+            connection.cleanup_pipe()
             # Establish the connection encryption 
             connection.connect_to_origin()
             # Propagate the connection to the original request
@@ -637,9 +678,9 @@
     def http_error_302(self, req, fp, code, msg, headers):
         """Requests the redirected to URI.
 
-        Copied from urllib2 to be able to fake_close the
-        associated connection, *before* issuing the redirected
-        request but *after* having eventually raised an error.
+        Copied from urllib2 to be able to clean the pipe of the associated
+        connection, *before* issuing the redirected request but *after* having
+        eventually raised an error.
         """
         # Some servers (incorrectly) return multiple Location headers
         # (so probably same goes for URI).  Use first header.
@@ -684,7 +725,7 @@
         # use it with HTTPError.
         fp.close()
         # We have all we need already in the response
-        req.connection.fake_close()
+        req.connection.cleanup_pipe()
 
         return self.parent.open(redirected_req)
 
@@ -1279,11 +1320,6 @@
     def http_error_default(self, req, fp, code, msg, hdrs):
         if code == 403:
             raise errors.TransportError('Server refuses to fullfil the request')
-        elif code == 416:
-            # We don't know which, but one of the ranges we
-            # specified was wrong. So we raise with 0 for a lack
-            # of a better magic value.
-            raise errors.InvalidRange(req.get_full_url(),0)
         else:
             raise errors.InvalidHttpResponse(req.get_full_url(),
                                              'Unable to handle http code %d: %s'

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2007-11-29 23:22:01 +0000
+++ bzrlib/transport/http/response.py	2007-12-10 10:41:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,272 +22,281 @@
 """
 
 
-from bisect import bisect
-from cStringIO import StringIO
-import re
-
-from bzrlib import errors
-from bzrlib.trace import mutter
-
-
-class ResponseRange(object):
-    """A range in a RangeFile-object."""
-
-    __slots__ = ['_ent_start', '_ent_end', '_data_start']
-
-    def __init__(self, ent_start, ent_end, data_start):
-        self._ent_start = ent_start
-        self._ent_end = ent_end
-        self._data_start = data_start
-
-    def __cmp__(self, other):
-        """Compare this to other.
-
-        We need this both for sorting, and so that we can
-        bisect the list of ranges.
-        """
-        if isinstance(other, int):
-            # Later on we bisect for a starting point
-            # so we allow comparing against a single integer
-            return cmp(self._ent_start, other)
-        else:
-            return cmp((self._ent_start, self._ent_end, self._data_start),
-                       (other._ent_start, other._ent_end, other._data_start))
-
-    def __str__(self):
-        return "%s(%s-%s,%s)" % (self.__class__.__name__,
-                                 self._ent_start, self._ent_end,
-                                 self._data_start)
-
-    __repr__ = __str__
-
+import httplib
+
+from bzrlib import (
+    errors,
+    trace,
+    )
+
+
+# A RangeFile expects the following grammar (simplified to outline the
+# assumptions we rely upon).
+
+# file: whole_file
+#     | single_range
+#     | multiple_range
+
+# whole_file: [content_length_header] data
+
+# single_range: content_range_header data
+
+# multiple_range: boundary_header boundary (content_range_header data boundary)+
 
 class RangeFile(object):
     """File-like object that allow access to partial available data.
 
-    Specified by a set of ranges.
+    All accesses should happen sequentially since the acquisition occurs during
+    an http response reception (as sockets can't be seeked, we simulate the
+    seek by just reading and discarding the data).
+
+    The access pattern is defined by a set of ranges discovered as reading
+    progress. Only one range is available at a given time, so all accesses
+    should happen with monotonically increasing offsets.
     """
 
-    def __init__(self, path, input_file):
+    def __init__(self, path, infile):
+        """Constructor.
+
+        :param path: File url, for error reports.
+        :param infile: File-like socket set at body start.
+        """
         self._path = path
-        self._pos = 0
-        self._len = 0
-        self._ranges = []
-        self._data = input_file.read()
-
-    def _add_range(self, ent_start, ent_end, data_start):
-        """Add an entity range.
-
-        :param ent_start: Start offset of entity
-        :param ent_end: End offset of entity (inclusive)
-        :param data_start: Start offset of data in data stream.
-        """
-        self._ranges.append(ResponseRange(ent_start, ent_end, data_start))
-        self._len = max(self._len, ent_end)
-
-    def _finish_ranges(self):
-        self._ranges.sort()
-
-    def read(self, size):
+        self._file = infile
+        self._boundary = None
+        # When using multi parts response, this will be set with the headers
+        # associated with the range currently read.
+        self._headers = None
+        # Default to the whole file of unspecified size
+        self.set_range(0, -1)
+
+    def set_range(self, start, size):
+        """Change the range mapping"""
+        self._start = start
+        self._size = size
+        # Set the new _pos since that's what we want to expose
+        self._pos = self._start
+
+    def set_boundary(self, boundary):
+        """Define the boundary used in a multi parts message.
+        
+        The file should be at the beginning of the body, the first range
+        definition is read and taken into account.
+        """
+        self._boundary = boundary
+        # Decode the headers and setup the first range
+        self.read_boundary()
+        self.read_range_definition()
+
+    def read_boundary(self):
+        """Read the boundary headers defining a new range"""
+        boundary_line = '\r\n'
+        while boundary_line == '\r\n':
+            # RFC2616 19.2 Additional CRLFs may precede the first boundary
+            # string entity.
+            # To be on the safe side we allow it before any boundary line
+            boundary_line = self._file.readline()
+        if boundary_line != '--' + self._boundary + '\r\n':
+            raise errors.InvalidHttpResponse(
+                self._path,
+                "Expected a boundary (%s) line, got '%s'" % (self._boundary,
+                                                             boundary_line))
+
+    def read_range_definition(self):
+        """Read a new range definition in a multi parts message.
+
+        Parse the headers including the empty line following them so that we
+        are ready to read the data itself.
+        """
+        self._headers = httplib.HTTPMessage(self._file, seekable=0)
+        # Extract the range definition
+        content_range = self._headers.getheader('content-range', None)
+        if content_range is None:
+            raise errors.InvalidHttpResponse(
+                self._path,
+                'Content-Range header missing in a multi-part response')
+        self.set_range_from_header(content_range)
+
+    def set_range_from_header(self, content_range):
+        """Helper to set the new range from its description in the headers"""
+        try:
+            rtype, values = content_range.split()
+        except ValueError:
+            raise errors.InvalidHttpRange(self._path, content_range,
+                                          'Malformed header')
+        if rtype != 'bytes':
+            raise errors.InvalidHttpRange(self._path, content_range,
+                                          "Unsupported range type '%s'" % rtype)
+        try:
+            # We don't need total, but note that it may be either the file size
+            # or '*' if the server can't or doesn't want to return the file
+            # size.
+            start_end, total = values.split('/')
+            start, end = start_end.split('-')
+            start = int(start)
+            end = int(end)
+        except ValueError:
+            raise errors.InvalidHttpRange(self._path, content_range,
+                                          'Invalid range values')
+        size = end - start + 1
+        if size <= 0:
+            raise errors.InvalidHttpRange(self._path, content_range,
+                                          'Invalid range, size <= 0')
+        self.set_range(start, size)
+
+    def _checked_read(self, size):
+        """Read the file checking for short reads"""
+        pos = self._pos
+        data = self._file.read(size)
+        data_len = len(data)
+        if size > 0 and data_len != size:
+            raise errors.ShortReadvError(self._path, pos, size, data_len)
+        self._pos += data_len
+        return data
+
+    def _seek_to_next_range(self):
+        # We will cross range boundaries
+        if self._boundary is None:
+            # If we don't have a boundary, we can't find another range
+            raise errors.InvalidRange(
+                self._path, self._pos,
+                "Range (%s, %s) exhausted"
+                % (self._start, self._size))
+        self.read_boundary()
+        self.read_range_definition()
+
+    def read(self, size=-1):
         """Read size bytes from the current position in the file.
 
-        Reading across ranges is not supported.
+        Reading across ranges is not supported. We rely on the underlying http
+        client to clean the socket if we leave bytes unread. This may occur for
+        the final boundary line of a multipart response or for any range
+        request not entirely consumed by the client (due to offset coalescing)
         """
-        # find the last range which has a start <= pos
-        i = bisect(self._ranges, self._pos) - 1
-
-        if i < 0 or self._pos > self._ranges[i]._ent_end:
-            mutter('Bisect for pos: %s failed. Found offset: %d, ranges:%s',
-                   self._pos, i, self._ranges)
-            raise errors.InvalidRange(self._path, self._pos)
-
-        r = self._ranges[i]
-
-        # mutter('found range %s %s for pos %s', i, self._ranges[i], self._pos)
-
-        if (self._pos + size - 1) > r._ent_end:
-            raise errors.InvalidRange(self._path, self._pos)
-
-        start = r._data_start + (self._pos - r._ent_start)
-        end   = start + size
-        # mutter("range read %d bytes at %d == %d-%d", size, self._pos,
-        #         start, end)
-        self._pos += (end-start)
-        return self._data[start:end]
+        if (self._size > 0
+            and self._pos == self._start + self._size):
+            if size == 0:
+                return ''
+            else:
+                self._seek_to_next_range()
+        elif self._pos < self._start:
+            raise errors.InvalidRange(
+                self._path, self._pos,
+                "Can't read %s bytes before range (%s, %s)"
+                % (size, self._start, self._size))
+        if self._size > 0:
+            if size > 0 and self._pos + size > self._start + self._size:
+                raise errors.InvalidRange(
+                    self._path, self._pos,
+                    "Can't read %s bytes across range (%s, %s)"
+                    % (size, self._start, self._size))
+
+        if self._size > 0:
+            # Don't read past the range definition
+            limited = self._start + self._size - self._pos
+            if size >= 0:
+                limited = min(limited, size)
+            data = self._file.read(limited)
+        else:
+            # Size of file unknown, the user may have specified a size or not,
+            # we delegate that to the filesocket object (-1 means read until
+            # EOF)
+            data = self._file.read(size)
+        # Update _pos respecting the data effectively read
+        self._pos += len(data)
+        return data
 
     def seek(self, offset, whence=0):
+        start_pos = self._pos
         if whence == 0:
-            self._pos = offset
+            final_pos = offset
         elif whence == 1:
-            self._pos += offset
+            final_pos = start_pos + offset
         elif whence == 2:
-            self._pos = self._len + offset
+            if self._size > 0:
+                final_pos = self._start + self._size + offset # offset < 0
+            else:
+                raise errors.InvalidRange(
+                    self._path, self._pos,
+                    "RangeFile: can't seek from end while size is unknown")
         else:
             raise ValueError("Invalid value %s for whence." % whence)
 
-        if self._pos < 0:
-            self._pos = 0
+        if final_pos < self._pos:
+            # Can't seek backwards
+            raise errors.InvalidRange(
+                self._path, self._pos,
+                'RangeFile: trying to seek backwards to %s' % final_pos)
+
+        if self._size > 0:
+            cur_limit = self._start + self._size
+            while final_pos > cur_limit:
+                # We will cross range boundaries
+                remain = cur_limit - self._pos
+                if remain > 0:
+                    # Finish reading the current range
+                    self._checked_read(remain)
+                self._seek_to_next_range()
+                cur_limit = self._start + self._size
+
+        size = final_pos - self._pos
+        if size > 0: # size can be < 0 if we crossed a range boundary
+            # We don't need the data, just read it and throw it away
+            self._checked_read(size)
 
     def tell(self):
         return self._pos
 
 
-class HttpRangeResponse(RangeFile):
-    """A single-range HTTP response."""
-
-    # TODO: jam 20060706 Consider compiling these regexes on demand
-    _CONTENT_RANGE_RE = re.compile(
-        r'\s*([^\s]+)\s+([0-9]+)-([0-9]+)/([0-9]+)\s*$')
-
-    def __init__(self, path, content_range, input_file):
-        # mutter("parsing 206 non-multipart response for %s", path)
-        RangeFile.__init__(self, path, input_file)
-        start, end = self._parse_range(content_range, path)
-        self._add_range(start, end, 0)
-        self._finish_ranges()
-
-    @staticmethod
-    def _parse_range(range, path='<unknown>'):
-        """Parse an http Content-range header and return start + end
-
-        :param range: The value for Content-range
-        :param path: Provide to give better error messages.
-        :return: (start, end) A tuple of integers
-        """
-        match = HttpRangeResponse._CONTENT_RANGE_RE.match(range)
-        if not match:
-            raise errors.InvalidHttpRange(path, range,
-                                          "Invalid Content-range")
-
-        rtype, start, end, total = match.groups()
-
-        if rtype != 'bytes':
-            raise errors.InvalidHttpRange(path, range,
-                    "Unsupported range type '%s'" % (rtype,))
-
-        try:
-            start = int(start)
-            end = int(end)
-        except ValueError, e:
-            raise errors.InvalidHttpRange(path, range, str(e))
-
-        return start, end
-
-
-class HttpMultipartRangeResponse(RangeFile):
-    """A multi-range HTTP response."""
-    
-    _CONTENT_TYPE_RE = re.compile(
-        r'^\s*multipart/byteranges\s*;\s*boundary\s*=\s*("?)([^"]*?)\1\s*$')
-    
-    # Start with --<boundary>\r\n
-    # and ignore all headers ending in \r\n
-    # except for content-range:
-    # and find the two trailing \r\n separators
-    # indicating the start of the text
-    # TODO: jam 20060706 This requires exact conformance
-    #       to the spec, we probably could relax the requirement
-    #       of \r\n, and use something more like (\r?\n)
-    _BOUNDARY_PATT = (
-        "^--%s(?:\r\n(?:(?:content-range:([^\r]+))|[^\r]+))+\r\n\r\n")
-
-    def __init__(self, path, content_type, input_file):
-        # mutter("parsing 206 multipart response for %s", path)
-        # TODO: jam 20060706 Is it valid to initialize a
-        #       grandparent without initializing parent?
-        RangeFile.__init__(self, path, input_file)
-
-        self.boundary_regex = self._parse_boundary(content_type, path)
-        # mutter('response:\n%r', self._data)
-
-        for match in self.boundary_regex.finditer(self._data):
-            ent_start, ent_end = HttpRangeResponse._parse_range(match.group(1),
-                                                                path)
-            self._add_range(ent_start, ent_end, match.end())
-
-        self._finish_ranges()
-
-    @staticmethod
-    def _parse_boundary(ctype, path='<unknown>'):
-        """Parse the Content-type field.
-        
-        This expects a multipart Content-type, and returns a
-        regex which is capable of finding the boundaries
-        in the multipart data.
-        """
-        match = HttpMultipartRangeResponse._CONTENT_TYPE_RE.match(ctype)
-        if not match:
-            raise errors.InvalidHttpContentType(path, ctype,
-                    "Expected multipart/byteranges with boundary")
-
-        boundary = match.group(2)
-        # mutter('multipart boundary is %s', boundary)
-        pattern = HttpMultipartRangeResponse._BOUNDARY_PATT
-        return re.compile(pattern % re.escape(boundary),
-                          re.IGNORECASE | re.MULTILINE)
-
-
-def _is_multipart(content_type):
-    return content_type.startswith('multipart/byteranges;')
-
-
-def handle_response(url, code, headers, data):
-    """Interpret the code & headers and return a HTTP response.
-
-    This is a factory method which returns an appropriate HTTP response
-    based on the code & headers it's given.
+def handle_response(url, code, msg, data):
+    """Interpret the code & headers and wrap the provided data in a RangeFile.
+
+    This is a factory method which returns an appropriate RangeFile based on
+    the code & headers it's given.
 
     :param url: The url being processed. Mostly for error reporting
     :param code: The integer HTTP response code
-    :param headers: A dict-like object that contains the HTTP response headers
+    :param msg: An HTTPMessage containing the headers for the response
     :param data: A file-like object that can be read() to get the
                  requested data
     :return: A file-like object that can seek()+read() the 
              ranges indicated by the headers.
     """
-
-    if code == 206:
-        try:
-            content_type = headers['Content-Type']
-        except KeyError:
-            # When there is no content-type header we treat
-            # the response as being of type 'application/octet-stream' as per
-            # RFC2616 section 7.2.1.
+    rfile = RangeFile(url, data)
+    if code == 200:
+        # A whole file
+        size = msg.getheader('content-length', None)
+        if size is None:
+            size = -1
+        else:
+            size = int(size)
+        rfile.set_range(0, size)
+    elif code == 206:
+        content_type = msg.getheader('content-type', None)
+        if content_type is None:
+            # When there is no content-type header we treat the response as
+            # being of type 'application/octet-stream' as per RFC2616 section
+            # 7.2.1.
             # Therefore it is obviously not multipart
             content_type = 'application/octet-stream'
             is_multipart = False
         else:
-            is_multipart = _is_multipart(content_type)
+            is_multipart = (msg.getmaintype() == 'multipart'
+                            and msg.getsubtype() == 'byteranges')
 
         if is_multipart:
             # Full fledged multipart response
-            return HttpMultipartRangeResponse(url, content_type, data)
+            rfile.set_boundary(msg.getparam('boundary'))
         else:
             # A response to a range request, but not multipart
-            try:
-                content_range = headers['Content-Range']
-            except KeyError:
+            content_range = msg.getheader('content-range', None)
+            if content_range is None:
                 raise errors.InvalidHttpResponse(url,
                     'Missing the Content-Range header in a 206 range response')
-            return HttpRangeResponse(url, content_range, data)
-    elif code == 200:
-        # A regular non-range response, unfortunately the result from
-        # urllib doesn't support seek, so we wrap it in a StringIO
-        tell = getattr(data, 'tell', None)
-        if tell is None:
-            return StringIO(data.read())
-        return data
-    elif code == 404:
-        raise errors.NoSuchFile(url)
-    # Some servers will retun "400: Bad Request" when too much ranges are
-    # specified
-    elif code in (400, 416):
-        # We don't know which, but one of the ranges we specified
-        # was wrong. So we raise with 0 for a lack of a better
-        # magic value.
-        raise errors.InvalidRange(url,0)
-
+            rfile.set_range_from_header(content_range)
     else:
-        raise errors.InvalidHttpResponse(url, "Unknown response code %s" 
-                                              % (code,))
+        raise errors.InvalidHttpResponse(url,
+                                         'Unknown response code %s' % code)
+
+    return rfile
 

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2007-12-06 21:39:48 +0000
+++ doc/en/mini-tutorial/index.txt	2007-12-17 01:33:01 +0000
@@ -34,7 +34,7 @@
 .. _installation instructions: http://bazaar-vcs.org/WindowsDownloads
 .. _install using MacPorts: http://bazaar-vcs.org/MacPorts
 .. _Downloads: http://bazaar-vcs.org/Download
-.. _Installation: http://bazaar-vcs.org/InstallationFAQ
+.. _Installation: http://bazaar-vcs.org/InstallationFaq
 
 
 Introducing yourself
@@ -103,7 +103,7 @@
 Making changes to your files
 ============================
 
-Let's change a file and commit those changes to your branch.
+Let's change a file and commit that change to your branch.
 
 Edit ``test1.txt`` in your favourite editor, then check what have you done::
 
@@ -162,13 +162,13 @@
 Bazaar will create a ``myproject`` directory on the remote server and
 push your branch to it.
  
+Now anyone can create their own copy of your branch by typing::
+
+ $ bzr branch http://www.example.com/myproject
+ 
 **Note:** to use sftp, you may need to install ``paramiko`` and
 ``pyCrypto``. See http://bazaar-vcs.org/InstallationFaq for details.
 
-Now anyone can create their own copy of your branch by typing::
-
- $ bzr branch http://www.example.com/myproject
- 
 
 Publishing your branch with Launchpad
 =====================================
@@ -264,7 +264,7 @@
 =============
 
 You can find out more about Bazaar in the
-`Bazaar user guide <../user-guide/index.html>`_.
+`Bazaar User Guide <../user-guide/index.html>`_.
 
 To learn about Bazaar on the command-line::
 

=== added file 'doc/en/user-guide/adv_merging.txt'
--- doc/en/user-guide/adv_merging.txt	1970-01-01 00:00:00 +0000
+++ doc/en/user-guide/adv_merging.txt	2007-12-14 07:35:49 +0000
@@ -0,0 +1,113 @@
+Pseudo merging
+==============
+
+Cherrypicking
+-------------
+
+At times, it can be useful to selectively merge some of the changes
+in a branch, but not all of them. This is commonly referred to as
+*cherrypicking*. Here are some examples of where cherrypicking is
+useful:
+
+* selectively taking fixes from the main development branch into
+  a release branch
+
+* selectively taking improvements out of an experimental branch into
+  a feature branch.
+
+To merge only the changes made by revision X in branch ``foo``,
+the command is::
+
+  bzr merge -c X foo
+
+To merge only the changes up to revision X in branch ``foo``,
+the command is::
+
+  bzr merge -r X foo
+
+To merge only the changes since revision X in branch ``foo``,
+the command is::
+
+  bzr merge -r X.. foo
+
+To merge only the changes from revision X to revision Y in branch ``foo``,
+the command is::
+
+  bzr merge -r X..Y foo
+
+Like a normal merge, you must explicitly commit a cherrypick. You may wish
+to see the changes made using ``bzr diff``, and run your test suite if any,
+before doing this.
+
+Unlike a normal merge, Bazaar does not currently track cherrypicks.
+In particular, the changes look like a normal commit and the (internal)
+revision history of the changes from the other branch is lost.
+In many cases where they are useful (see above), this is not a major
+problem because there are good reasons why a full merge should never
+be done at a later time. In other cases, additional conflicts will need
+to be resolved when the changes are merged again.
+
+
+Reverse cherrypicking
+---------------------
+
+Cherrypicking can be used to reverse a set of changes made by giving an
+upper bound in the revision range which is *below* the lower bound.
+For example, to back-out changes made in revision 10, the command is::
+
+  bzr merge -r 10..9
+
+If you want to take most changes, but not all, from somewhere else, you
+may wish to do a normal merge followed by a few reverse cherrypicks.
+
+
+Merging uncommitted changes
+---------------------------
+
+If you have several branches and you accidently start making changes in the
+wrong one, here are the steps to take to correct this. Assuming you began
+working in branch ``foo`` when you meant to work in branch ``bar``:
+
+1. Change into branch ``bar``.
+2. Run ``bzr merge --uncommitted foo``
+3. Check the changes came across (``bzr diff``)
+4. Change into branch ``foo``
+5. Run ``bzr revert``.
+
+.. TODO Selective file merging?
+
+
+Rebasing
+--------
+
+Another option to normal merging is *rebasing*, i.e. making it look like
+the current branch originated from a different point than it did.
+Rebasing is supported in Bazaar by the ``rebase`` command provided by
+the ``rebase`` plugin.
+
+The ``rebase`` command takes the location of another branch on which
+the branch in the current working directory will be rebased. If a branch
+is not specified then the parent branch is used, and this is usually the
+desired result.
+
+The first step identifies the revisions that are in the current branch
+that are not in the parent branch. The current branch is then set to be
+at the same revision as the target branch, and each revision is replayed
+on top of the branch. At the end of the process it will appear as though
+your current branch was branched off the current last revision of the target.
+
+Each revision that is replayed may cause conflicts in the tree. If this
+happens the command will stop and allow you to fix them up. Resolve the
+commits as you would for a ``merge``, and then run ``bzr resolve`` to
+marked them as resolved. Once you have resolved all the conflicts, you
+should run ``bzr rebase-continue`` to continue the rebase operation.
+If conflicts are encountered and you decide not to continue,
+you can run ``bzr rebase-abort``. You can also use ``rebase-todo`` to
+show the list of commits still to be replayed.
+
+Note: Some users coming from central VCS tools with poor merge tracking
+like rebasing because it's similar to how they are use to working in older
+tools, or because "perfectly clean" history seems important. Before rebasing
+in Bazaar, think about whether a normal merge is a better choice. In
+particular, rebasing a private branch before sharing it is OK but
+rebasing after sharing a branch with someone else is **strongly** discouraged.

=== modified file 'doc/en/user-guide/bug_trackers.txt'
--- doc/en/user-guide/bug_trackers.txt	2007-12-04 18:11:51 +0000
+++ doc/en/user-guide/bug_trackers.txt	2007-12-13 13:33:36 +0000
@@ -2,95 +2,53 @@
 ====================================
 
 Bazaar has a facility that allows you to associate a commit with a bug
-in the project's bug tracker. Other tools can then use this information
-to link between the commit and the bug, or to automatically mark the bug
-closed in the branches that contain the commit.
+in the project's bug tracker. Other tools (or hooks) can then use this
+information to link between the commit and the bug, or to automatically
+mark the bug closed in the branches that contain the commit.
 
 Associating commits and bugs
 ----------------------------
 
-When you make a commit you can assiociate it with a bug by using the
+When you make a commit, you can associate it with a bug by using the
 ``--fixes`` option of ``commit``. For example::
 
-    $ bzr commit --fixes project:12345 -m "Properly close the connection"
-
-This will set a revision property on the revision that is committed
-which contains the URI at which more information on the bug can be found.
-Bazaar itself does no more than this, but once the information is recorded
-other tools can make use of it.
-
-Configuration of the bug tracker
---------------------------------
-
-As each project has it's own bug tracker, and there are many different
-bug tracking systems you must first do some configuration before you
-can use this system.
-
-Bazaar currently implements support for projects that use Bugzilla or
-Trac to manage their bugs. If your project uses a different system it
-is easy to add support for the system, perhaps through a plugin.
-
-If you do use Bugzilla or Trac then you only need to set a configuration
-variable which contains the base URL of the bug tracker. These options
-can go into ``bazaar.conf``, ``branch.conf`` or into a branch-specific
-configuration section in ``locations.conf``.
-
-You can set up these values for each of the projects you work on in
-``bazaar.conf``, as you specify a short name for each so that you
-can specify which you mean at commit time.
-
-bugzilla_<tracker_abbreviation>_url
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-If present, the location of the Bugzilla bug tracker referred to by
-<tracker_abbreviation>. This option can then be used together with ``bzr commit
---fixes`` to mark bugs in that tracker as being fixed by that commit. For
-example::
-
-    bugzilla_squid_url = http://www.squid-cache.org/bugs
-
-would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as
-fixed.
-
-trac_<tracker_abbrevation>_url
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-If present, the location of the Trac instance referred to by
-<tracker_abbreviation>. This option can then be used together with ``bzr commit
---fixes`` to mark bugs in that tracker as being fixed by that commit. For
-example::
-
-    trac_twisted_url = http://www.twistedmatrix.com/trac
-
-would allow ``bzr commit --fixes twisted:1234`` to mark Twisted's bug 1234 as
-fixed.
-
-bugtracker_<tracker_abbrevation>_url
-------------------------------------
-If present, the location of a generic bug tracker instance referred to by
-<tracker_abbreviation>. The location must contain an ``{id}`` placeholder,
-which will be replaced by a specific bug ID. This option can then be used
-together with ``bzr commit --fixes`` to mark bugs in that tracker as being
-fixed by that commit. For example::
-
-    bugtracker_python_url = http://bugs.python.org/issue{id}
-
-would allow ``bzr commit --fixes python:1234`` to mark bug 1234 in Python's
-Roundup bug tracker as fixed, or::
-
-    bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}
-
-for CPAN's RT bug tracker.
-
-Limitations of the system
--------------------------
+    $ bzr commit --fixes lp:12345 -m "Properly close the connection"
+
+This records metadata in Bazaar linking the commit with bug 12345 in
+Launchpad. If you use a different bug tracker, it can be given its own
+tracker code (instead of ``lp``) and used instead. For details on how
+to configure this, refer to `Bug Tracker Settings`_ in the Bazaar
+User Reference.
+
+.. _Bug Tracker Settings: ../user-reference/bzr_man.html#bug-tracker-settings
+
+Metadata recording vs bug tracker updating
+------------------------------------------
+
+Recording metadata about bugs fixed at commit time is only
+one of the features needed for complete bug tracker integration.
+As Bazaar is a distributed VCS, users may be offline while committing
+so accessing the bug tracker itself at that time may not be possible.
+Instead, it is recommended that a hook be installed to update
+the bug tracker when changes are pushed to a central location
+appropriate for your project's workflow.
+
+Note: This second processing stage is part of the integration provided
+by Lauchpad when it scans external or hosted branches.
+
+Making corrections
+------------------
 
 This method of associating revisions and bugs does have some limitations. The
 first is that the association can only be made at commit time. This means that
 if you forget to make the association when you commit, or the bug is reported
-after you fix it you cannot go back and add the link later.
+after you fix it, you generally cannot go back and add the link later.
 
 Related to this is the fact that the association is immutable. If a bug is
-marked as fixed by one commit, but that revision does not fully solve the
-bug, or there is a later regression you cannot go back and remove the link.
+marked as fixed by one commit but that revision does not fully solve the
+bug, or there is a later regression, you cannot go back and remove the link.
 
+Of course, ``bzr uncommit`` can always be used to undo the last commit in
+order to make it again with the correct options. This is commonly done
+to correct a bad commit message and it equally applies to correcting
+metadata recorded (via ``--fixes`` for example) on the last commit.

=== modified file 'doc/en/user-guide/core_concepts.txt'
--- doc/en/user-guide/core_concepts.txt	2007-12-04 11:51:38 +0000
+++ doc/en/user-guide/core_concepts.txt	2007-12-14 07:35:49 +0000
@@ -38,6 +38,9 @@
 While these identifiers are necessary for internal use and external tool
 integration, branch-specific *revision numbers* are the preferred
 interface for humans. Typical revision numbers are 1, 42 and 2977.1.59.
+See `Specifying revisions`_ in the appendices for a closer look at
+the numerous ways that revisions and ranges of revisions can be specified
+in Bazaar.
 
 .. *TODO: add diagram*
 

=== modified file 'doc/en/user-guide/http_smart_server.txt'
--- doc/en/user-guide/http_smart_server.txt	2007-11-14 03:50:56 +0000
+++ doc/en/user-guide/http_smart_server.txt	2007-12-14 07:35:49 +0000
@@ -10,7 +10,7 @@
 For more information on the smart server, and other ways to configure it
 see the main `smart server documentation`_.
 
-.. _smart server documentation: server.html
+.. _smart server documentation: #running-a-smart-server
 
 Example
 -------

=== modified file 'doc/en/user-guide/index.txt'
--- doc/en/user-guide/index.txt	2007-12-07 03:37:32 +0000
+++ doc/en/user-guide/index.txt	2007-12-17 01:33:01 +0000
@@ -79,11 +79,11 @@
 ##############
 
 .. include:: best_practice_intro.txt
+.. include:: adv_merging.txt
 .. include:: server.txt
 .. include:: hooks.txt
 .. include:: version_info.txt
 .. include:: bug_trackers.txt
-.. include:: shared_repository_layouts.txt
  
 
 Appendices

=== modified file 'doc/en/user-guide/installing_bazaar.txt'
--- doc/en/user-guide/installing_bazaar.txt	2007-11-30 04:28:32 +0000
+++ doc/en/user-guide/installing_bazaar.txt	2007-12-14 07:35:49 +0000
@@ -6,7 +6,7 @@
 
 Bazaar packages are available for most popular Linux distributions
 including Ubuntu/Debian, Red Hat and Gentoo.
-See http://bazaar-vcs.org/Downloads for the latest instructions.
+See http://bazaar-vcs.org/Download for the latest instructions.
 
 Windows
 -------
@@ -14,7 +14,7 @@
 For Windows users, an installer is available that includes
 the core Bazaar package together with necessary pre-requisites
 and some useful plug-ins.
-See http://bazaar-vcs.org/Downloads for the latest instructions.
+See http://bazaar-vcs.org/Download for the latest instructions.
 
 Note: If you are running Cygwin on Windows, a Bazaar for Cygwin package
 is available and ought to be used instead of the Windows version.
@@ -24,7 +24,7 @@
 
 Beyond Linux and Windows, Bazaar packages are available for a large
 range of other operating systems include Mac OS X, FreeBSD and Solaris.
-See http://bazaar-vcs.org/Downloads for the latest instructions.
+See http://bazaar-vcs.org/Download for the latest instructions.
 
 
 Installing from scratch
@@ -36,7 +36,7 @@
   1. If it is not installed already, install Python 2.4 or later.
 
   2. Download the ``bazaar-xxx.tar.gz`` file (where xxx is the version
-     number) from http://bazaar-vcs.org/Downloads or from Launchpad
+     number) from http://bazaar-vcs.org/Download or from Launchpad
      (https://launchpad.net/~bzr/).
 
   3. Unpack the archive using tar, WinZip or equivalent.

=== modified file 'doc/en/user-guide/plugins.txt'
--- doc/en/user-guide/plugins.txt	2007-11-30 04:28:32 +0000
+++ doc/en/user-guide/plugins.txt	2007-12-14 07:35:49 +0000
@@ -34,11 +34,11 @@
 -----------------------
 
 Installing a plugin is very easy! If not already created, create a
-``plugins`` directory under your Bazaar configuration directory
-($BZR_HOME), i.e. ``~/.bazaar/`` on Linux and
+``plugins`` directory under your Bazaar configuration directory,
+``~/.bazaar/`` on Linux and
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0\``
-on Windows. Within this directory, each plugin is placed in its own
-subdirectory.
+on Windows. Within this directory (referred to as $BZR_HOME below),
+each plugin is placed in its own subdirectory.
 
 Plugins work particularly well with Bazaar branches. For example, to
 install the bzrtools plugins for your main user account on Linux,
@@ -47,11 +47,11 @@
     bzr branch http://panoramicfeedback.com/opensource/bzr/bzrtools
     ~/.bazaar/plugins/bzrtools
 
-When installing plugins the directories that you install them in must
+When installing plugins, the directories that you install them in must
 be valid python identifiers. This means that they can only contain
 certain characters, notably they cannot contain hyphens (``-``). Rather
-than installing ``bzr-gtk`` to ``~/.bazaar/plugins/bzr-gtk``, install it
-to ``~/.bazaar/plugins/gtk``.
+than installing ``bzr-gtk`` to ``$BZR_HOME/plugins/bzr-gtk``, install it
+to ``$BZR_HOME/plugins/gtk``.
 
 Alternative plugin locations
 ----------------------------

=== modified file 'doc/en/user-guide/publishing_a_branch.txt'
--- doc/en/user-guide/publishing_a_branch.txt	2007-11-23 05:52:03 +0000
+++ doc/en/user-guide/publishing_a_branch.txt	2007-12-14 07:35:49 +0000
@@ -17,7 +17,11 @@
   bzr init-repo --no-trees sftp://centralhost/srv/bzr/X-repo/
 
 You can think of this step as similar to setting up a new cvsroot or
-Subversion repository.
+Subversion repository. However, Bazaar gives you more flexibility
+in how branches may be organised in your repository. See
+`Choosing a shared repository layout`_ in the appendices for
+guidelines and examples.
+
 
 Starting a central branch
 -------------------------

=== modified file 'doc/en/user-guide/reusing_a_checkout.txt'
--- doc/en/user-guide/reusing_a_checkout.txt	2007-12-04 12:56:31 +0000
+++ doc/en/user-guide/reusing_a_checkout.txt	2007-12-14 07:35:49 +0000
@@ -20,6 +20,7 @@
 configured into numerous development and testing tools. Once
 again, reusing a checkout across multiple branches can help.
 
+
 Changing where a branch is bound to
 -----------------------------------
 
@@ -34,28 +35,28 @@
  3. Make your checkout a copy of the desired branch by using
     the ``update`` command followed by the ``revert`` command.
 
-Note that simply binding to a new branch
-and running ``update`` merges in your local changes. You need
+Note that simply binding to a new branch and running ``update``
+merges in your local changes, both committed and uncommitted. You need
 to decide whether to keep them or not by running either ``revert``
 or ``commit``.
 
-An alternative to the bind+update recipe is:
-
- 1. Use the ``unbind`` command to turn the checkout into a normal branch.
-
- 2. Run ``bzr pull --overwrite URL`` to make the branch a mirror of
-    a new URL.
-
- 3. Run ``bzr bind URL`` to bind to that URL.
+An alternative to the bind+update recipe is using the ``switch``
+command. This is basically the same as removing the existing
+branch and running ``checkout`` again on the new location, except
+that any uncommitted changes in your tree are merged in.
+
+Note: As ``switch`` can potentially throw away committed changes in
+order to make a checkout an accurate cache of a different bound branch,
+it will fail by design if there are changes which have been committed
+locally but are not yet committed to the most recently bound branch.
+To truly abandon these changes, use the ``--force`` option.
+
 
 Switching a lightweight checkout
 --------------------------------
 
-If you have a lightweight checkout, there is no local history
-to worry about. In this case, you
-can simply alter the branch that the checkout (i.e.
-working tree) is referring to by using the ``switch`` command.
-
+With a lightweight checkout, there are no local commits and ``switch``
+effectively changes which branch the working tree is associated with.
 One possible setup is to use a lightweight checkout in combination
 with a local tree-less repository. This lets you switch what you
 are working on with ease. For example::
@@ -80,15 +81,6 @@
   bzr switch ../X-trunk
   (go back to working on the trunk)
 
-In many ways, ``switch`` operates like ``update`` in that it
-refreshes your working tree and merges in any local changes you
-have made. The primary different is that ``switch`` requires
-a branch location and it is only supported (currently) on
-lightweight checkouts. ``switch`` also only preserves
-uncommitted changes while update keeps committed ones.
-
 Note: The branches may be local only or they may be bound to
 remote ones (by creating them with ``checkout`` or by using ``bind``
-after creating them with ``branch``). ``switch`` and ``bind`` are
-similar but different: switch controls which branch the working tree
-refers to while ``bind`` associates a remote branch to the local one.
+after creating them with ``branch``).

=== modified file 'doc/en/user-guide/server.txt'
--- doc/en/user-guide/server.txt	2007-11-14 03:50:56 +0000
+++ doc/en/user-guide/server.txt	2007-12-14 07:35:49 +0000
@@ -1,5 +1,5 @@
-Running a Bazaar server
-=======================
+Running a smart server
+======================
 
 Bazaar does not require a specialised server because it operates over HTTP, FTP
 or SFTP.  There is an optional smart server that can be invoked over SSH, from
@@ -17,22 +17,21 @@
 
 Bazaar supports writing over FTP, SFTP and (via a plugin) over HTTP-WebDAV.
 
-High-performance server
------------------------
+High-performance smart server
+-----------------------------
 
-The high-performance server is currently in development.  It provides a dumb
-server facility as well as additional capabilities which allow the version of
-bzr that accompanies this documentation to perform some operations much faster
-than via dumb servers.  In future releases of bzr the range of operations that
-are improved by using the high-performance server will increase.
+The high-performance smart server (hpss) performs certain operations much faster
+than dumb servers are capable of.  In future releases, the range of operations
+that are improved by using the smart server will increase as we continue to
+tune performance.
 
 To maintain the highest security possible, the current
-high-performance server provides read-only access by default.  To
+smart server provides read-only access by default.  To
 enable read-write access, run it with ``--allow-writes``. When using
 the SSH access method, bzr automatically runs with with the
 ``--allow-writes`` option.
 
-The server can be configured to run in three different ways:
+The alternative ways of configuring a smart server are explained below.
 
 SSH
 ~~~

=== modified file 'doc/en/user-guide/setting_up_email.txt'
--- doc/en/user-guide/setting_up_email.txt	2007-11-27 21:34:11 +0000
+++ doc/en/user-guide/setting_up_email.txt	2007-12-14 07:35:49 +0000
@@ -30,13 +30,13 @@
 
 The order of precedence is
 
- 1. If the ``$BZR_EMAIL`` environment variable is set.
+ 1. If the ``BZR_EMAIL`` environment variable is set.
  #. If an email is set for your current branch in the ``locations.conf``
     file.
  #. If an email is set four your current branch in the
     ``.bzr/branch/branch.conf`` file.
  #. If an email is set in the ``bazaar.conf`` default configuration file.
- #. If the `$EMAIL` environment variable is set.
+ #. If the `EMAIL` environment variable is set.
  #. Bazaar will try to guess based on your username and the hostname.
 
 To check on what Bazaar thinks your current email is, use the ``whoami``
@@ -61,24 +61,23 @@
 Setting email via default configuration file
 --------------------------------------------
 
-To use the default ini file, create the file ``$HOME/.bazaar/bazaar.conf`` (on
-Windows this is ``%APPDATA%\bazaar\2.0\bazaar.conf``) and set an email address
-as shown below.  Please note that the word DEFAULT is case sensitive, and
-must be in upper-case.
+To use the default ini file, create or edit the ``bazaar.conf`` file (in
+``~/.bazaar/`` on Linix and in ``%APPDATA%\bazaar\2.0\`` in Windows)
+and set an email address as shown below.  Please note that the word DEFAULT
+is case sensitive, and must be in upper-case.
 ::
 
     [DEFAULT]
     email=Your Name <name@isp.com>
 
 
-For more information on the ini file format, see ConfiguringBzr.
+For more information on the ini file format, see `Configuration reference`_.
 
 Setting email on a per-branch basis
 -----------------------------------
 
 The second approach is to set email on a branch by branch basis by
-using the ``$HOME/.bazaar/locations.conf`` configuration file,
-thusly::
+using the ``locations.conf`` configuration file like this::
 
     [/some/branch/location]
     email=Your Name <name@other-isp.com>
@@ -88,8 +87,8 @@
 
 Setting email via environment variable
 --------------------------------------
-The final method Bazaar will use is checking for the ``$BZR_EMAIL``
-and ``$EMAIL`` environment variables.  Generally, you would use this
+The final method Bazaar will use is checking for the ``BZR_EMAIL``
+and ``EMAIL`` environment variables.  Generally, you would use this
 method to override the email in a script context.  If you would like
 to set a general default, then please see the ini methods above.
 

=== modified file 'doc/en/user-guide/specifying_revisions.txt'
--- doc/en/user-guide/specifying_revisions.txt	2007-11-23 06:06:43 +0000
+++ doc/en/user-guide/specifying_revisions.txt	2007-12-14 07:35:49 +0000
@@ -1,13 +1,12 @@
-Revision specifiers
-===================
-
-Revision specs and ranges
--------------------------
-
-``bzr`` has a very expressive way to specify a revision, or a range of revisions.
-We'll take the example of the ``log`` command.
-
-To specify a range of revisions, use for example::
+Specifying revisions
+====================
+
+Revision identifiers and ranges
+-------------------------------
+
+Bazaar has a very expressive way to specify a revision or a range of revisions.
+To specify a range of revisions, the upper and lower bounds are separated by the
+``..`` symbol. For example::
 
     $ bzr log -r 1..4
 
@@ -16,17 +15,21 @@
     $ bzr log -r 1..
     $ bzr log -r ..4
 
-Note:
-    Omitting the lower bound doesn't work on versions of ``bzr`` prior to 0.14.
-
-Other commands, like ``bzr cat`` take only one revision, not a range, like::
+Some commands take only one revision, not a range. For example::
 
     $ bzr cat -r 42 foo.c
 
-Available revision specs
-------------------------
-
-The revision, or the bounds of the range, can be one of
+In other cases, a range is required but you want the length of the range to
+be one. For commands where this is relevant, the ``-c`` option is used like this::
+
+    $ bzr diff -c 42
+
+
+Available revision identifiers
+------------------------------
+
+The revision, or the bounds of the range, can be given using
+different format specifications as shown below.
 
  +----------------------+------------------------------------+
  |  argument type       | description                        |
@@ -43,10 +46,20 @@
  +----------------------+------------------------------------+
  | **date**:*value*     | first entry after a given date     |
  +----------------------+------------------------------------+
+ | **tag**:*value*      | revision matching a given tag      |
+ +----------------------+------------------------------------+
  | **ancestor**:*path*  | last merged revision from a branch |
  +----------------------+------------------------------------+
  | **branch**:*path*    | latest revision on another branch  |
  +----------------------+------------------------------------+
+ | **submit**:*path*    | common ancestor with submit branch |
+ +----------------------+------------------------------------+
+
+A brief introduction to some of these formats is given below.
+For complete details, see `Revision Identifiers`_ in the
+Bazaar User Reference.
+
+.. _Revision Identifiers: ../user-reference/bzr_man.html#revision-identifiers
 
 Numbers
 ~~~~~~~
@@ -64,15 +77,6 @@
 
     $ bzr log -r -10..
 
-revno, last
-~~~~~~~~~~~
-
-**revno**:*number*
-    The same as *number*, except that negative numbers are not allowed.
-
-**last**:*number*
-    The same as -''number''. **last:1** means the last commited revision.
-
 revid
 ~~~~~
 
@@ -83,7 +87,6 @@
 
     $ bzr log -r revid:Matthieu.Moy@imag.fr-20051026185030-93c7cad63ee570df
 
-
 before
 ~~~~~~
 

=== modified file 'doc/en/user-guide/version_info.txt'
--- doc/en/user-guide/version_info.txt	2007-11-21 02:54:47 +0000
+++ doc/en/user-guide/version_info.txt	2007-12-14 07:35:49 +0000
@@ -1,24 +1,51 @@
 Exporting version information
 =============================
 
-Overview
---------
-
-This document describes ways of using ``bzr version-info`` as part of a
-build process to embed version information into a final project.
-
-
-Python project
---------------
-
-TODO: Figure out how to attach into ``setup.py``
+Getting the last revision number
+--------------------------------
+
+If you only need the last revision number in your build scripts, you can
+use the ``revno`` command to get that value like this::
+
+  $ bzr revno
+  3104
+
+
+Getting more version information
+--------------------------------
+
+The ``version-info`` command can be used to output more information
+about the lastest version like this::
+
+  $ bzr version-info
+  revision-id: pqm@pqm.ubuntu.com-20071211175118-s94sizduj201hrs5
+  date: 2007-12-11 17:51:18 +0000
+  build-date: 2007-12-13 13:14:51 +1000
+  revno: 3104
+  branch-nick: bzr.dev
+
+You can easily filter that output using operating system tools or
+scripts. For example (on Linux/Unix)::
+
+  $ bzr version-info | grep ^date
+  date: 2007-12-11 17:51:18 +0000
+
+The ``--all`` option will actually dump version information about
+every revision if you need that information for more advanced
+post-processing.
+
+
+Python projects
+---------------
+
+.. TODO: Figure out how to attach into ``setup.py``
 
 
 If using a Makefile to build your project, you can generate the version
 information file as simply as::
 
   library/_version.py:
-        bzr version-info --format=python > library/_version.py
+        bzr version-info --format python > library/_version.py
 
 This generates a file which contains 3 dictionaries:
 
@@ -39,6 +66,16 @@
     map. This is also empty by default, and enabled only by ``--all``
     or ``--include-file-revisions``.
 
+
+Projects in other languages
+---------------------------
+
+Support for dumping version information in other languages in currently
+in development. Please contact us on the mailing list about your
+requirements in this area. (As of the Bazaar 1.0 release date, some patches
+are available for other languages but are not yet merged into the core product.)
+
+
 Check clean
 -----------
 

=== added file 'index.txt'
=== modified file 'setup.py'
--- setup.py	2007-12-11 02:07:30 +0000
+++ setup.py	2007-12-17 01:33:01 +0000
@@ -237,7 +237,7 @@
         for root, dirs, files in os.walk('doc'):
             r = []
             for f in files:
-                if os.path.splitext(f)[1] in ('.html', '.css'):
+                if os.path.splitext(f)[1] in ('.html','.css','.png','.pdf'):
                     r.append(os.path.join(root, f))
             if r:
                 relative = root[4:]

=== modified file 'tools/rst2html.py'
--- tools/rst2html.py	2007-08-07 14:56:50 +0000
+++ tools/rst2html.py	2007-12-07 06:54:13 +0000
@@ -1,24 +1,39 @@
-#!/usr/bin/env python
+#! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@python.org
-# Revision: $Revision: 3901 $
-# Date: $Date: 2005-09-25 17:49:54 +0200 (Sun, 25 Sep 2005) $
-# Copyright: This module has been placed in the public domain.
+# Originally by Dave Goodger, from the docutils, distribution.
+#
+# Modified for Bazaar to accommodate options containing dots
+#
+# This file is in the public domain.
 
 """
 A minimal front end to the Docutils Publisher, producing HTML.
 """
 
+# modified version for 
+
 try:
     import locale
     locale.setlocale(locale.LC_ALL, '')
 except:
     pass
 
+import docutils
 from docutils.core import publish_cmdline, default_description
 
 
+if docutils.__version__ <= '0.4.1':
+    from docutils.parsers.rst.states import Body
+    # we have some option names that contain dot; which is not allowed by
+    # python-docutils 0.4-4 -- so monkeypatch in a better pattern
+    #
+    # This is a bit gross to patch because all this is built up at load time.
+    Body.pats['optname'] = r'[a-zA-Z0-9][a-zA-Z0-9._-]*'    
+    Body.pats['longopt'] = r'(--|/)%(optname)s([ =]%(optarg)s)?' % Body.pats
+    Body.pats['option'] = r'(%(shortopt)s|%(longopt)s)' % Body.pats
+    Body.patterns['option_marker'] = r'%(option)s(, %(option)s)*(  +| ?$)' % Body.pats
+
+
 description = ('Generates (X)HTML documents from standalone reStructuredText '
                'sources.  ' + default_description)
 

